﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Net;
using System.Threading;
using Microsoft.DirectX.DirectSound;
using Microsoft.DirectX.DirectInput;
using System.Diagnostics;
using System.Xml;

namespace WindowsFormsApplication1
{

    public partial class MainForm : Form
    {
        private cameraSetting[] camSettings;                                        // 各カメラの設定
        private Cam_Feature_Data[] camFeat;                                         // 各カメラの機能
        private Cam_Status[] camState;                                              // 各カメラの状態
        private Cam_AutoStatus[] camAutoState;                                      // 各カメラの状態(明るさ補正など)
        private pathS path_;                                                        // 各種パス(ディレクトリ)
        private bool[] recFlags;                                                    // 各カメラのデータ受信フラグ
        private CookieContainer[] cookies;                                          //各カメラのセッション(クッキー)
        private IntPtr hwnd;                                                        // ウィンドウハンドル
        private int selCam;                                                         // 選択されているカメラ（の番号)
        private bool isContRun= false;                                              // ジョイパッドのスレッドが動作しているか
        private Thread thSession, thVideo, thAudio, thControl, thThumb, thStatus;   // スレッド(セッション更新、映像受信、音声受信、コントローラ入力、カメラ状態取得)
        private int num_of_preset;                                                  // プリセットの保存数
        private PresetData_with_Name[] presets;                                     // プリセットの情報を格納する配列

        private Microsoft.DirectX.DirectInput.Device joystick;                      // ゲームパッド
        private int selected_GP = 0;                                                // 選択されているゲームパッドの番号
        private int num_of_gamepad =0;                                              // パッドの数        
        private pads padKey;                                                        // パッドのキー割り当て

        private ptzClass ptzC;

        ///// Form1の値を他のフォームから呼び出すための記述 /////
        private static MainForm _form1Instance;
        public static MainForm Form1Instance { get { return _form1Instance; } set {_form1Instance = value; } }

        ///// フォーム上の値 /////////////////////////////////////
        public string userName { get { return textBox_USER.Text; } set { textBox_USER.Text = value; } }                                 // ユーザー名 //////////////////////////////////////////
        public string passWord { get { return textBox_PASS.Text; } set { textBox_PASS.Text = value; } }                                 // パスワード //////////////////////////////////////////
        public string ipAddr { get { return testBox_IP.Text; } set { testBox_IP.Text = value; } }                                       // 接続先IPアドレス ////////////////////////////////////
        public string portNumber { get { return port.Text; } set { port.Text = value; } }                                               // ポート番号 //////////////////////////////////////////
        public bool useProxy { get { return checkBox_useProxy.Checked; } set { checkBox_useProxy.Checked = value; } }                   // プロキシを使用するかどうか ///////////////////////////
        public bool inversePicture { get { return checkBox_InversePicture.Checked; } set { checkBox_InversePicture.Checked = value; } } // 映像を反転表示させるかどうか /////////////////////////
        public bool playSound { get { return checkBox_playSound.Checked; } set { checkBox_playSound.Checked = value; } }                // 音声を出力するかどうか ///////////////////////////////
        public bool overlayString { get { return checkBox_overlayText.Checked; } set { checkBox_overlayText.Checked = value; } }        // 文字を重畳させるかどうか /////////////////////////////
        public string overlayStringData { get { return textBox_overlayText.Text; } set { textBox_overlayText.Text = value; } }          // 重畳させる文字列 ////////////////////////////////////
        public int zoom_Speed { get { return (int)zoomSpeed.Value; } set { zoomSpeed.Value = (decimal)value; } }                        // ズームスピード (1～3) ///////////////////////////////
        public int pantilt_Speed { get { return (int)pantiltSpeed.Value; } set { pantiltSpeed.Value = (decimal)value; } }               // パンチルトスピード (1～3) ///////////////////////////
        public ListBox presetList { get { return listBox_Preset; } }                                                                    // プリセット一覧のリストボックス ///////////////////////
        public string presetList_Add { set { listBox_Preset.Items.Add(value); } }                                                       // プリセット一覧に項目を追加する ///////////////////////
        public string selectedPresetText { get { return listBox_Preset.Text; } set { listBox_Preset.Items[listBox_Preset.SelectedIndex] = value; } } // 現在選択されているプリセットの値 ////////
        public int selectedPresetIndex { get { return listBox_Preset.SelectedIndex; } }                                                 // 現在選択されているプリセットの番号 ///////////////////

        ///// 各カメラ個別に保持する値 ////////////////////////////
        public CookieContainer[] cookieContainers { get { return cookies; } set { cookies = value; } }                                  // クッキー ////////////////////////////////////////////
        public bool[] receiveFlags { get { return recFlags; } set { recFlags = value; } }                                               // 現在データを受信中かどうか ///////////////////////////
        public cameraSetting[] cameraSettings { get { return camSettings; } set { camSettings = value; } }                              // カメラ設定値 ///////////////////////////////////////
        public Cam_Feature_Data[] camFeatures { get { return camFeat; } set { camFeat = value; } }                                      // カメラ機能 /////////////////////////////////////////
        public Cam_Status[] camStatuses { get { return camState; } set { camState = value; } }                                          // カメラ状態 /////////////////////////////////////////      
        public Cam_AutoStatus[] camAutoStatuses { get { return camAutoState; } set { camAutoState = value; } }                          // カメラ状態(明るさ補正など) //////////////////////////

        ///// 全カメラ共通で扱う値 ///////////////////////////////
        public pathS pathes { get { return path_; } set { path_ = value; } }                                                            // 各種ディレクトリ ////////////////////////////////////
        public IntPtr handler { get { return hwnd; } }                                                                                  // MainFormのウィンドウハンドラ ////////////////////////
        public Image screen { get { return pictureBox1.Image; } set { pictureBox1.Image = value; } }                     // メイン画面 /////////////////////////////////////////
        public Image thumbCam1 { get { return pictureBox_cam1.Image; } set { pictureBox_cam1.Image = value; } }          // カメラ1のサムネイル /////////////////////////////////
        public Image thumbCam2 { get { return pictureBox_cam2.Image; } set { pictureBox_cam2.Image = value; } }          // カメラ2のサムネイル /////////////////////////////////
        public Image thumbCam3 { get { return pictureBox_cam3.Image; } set { pictureBox_cam3.Image = value; } }          // カメラ3のサムネイル /////////////////////////////////
        public Image thumbCam4 { get { return pictureBox_cam4.Image; } set { pictureBox_cam4.Image = value; } }          // カメラ4のサムネイル /////////////////////////////////
        public Image picturebox_cameraStatus { get { return pictureBox_State.Image; } set { pictureBox_State.Image = value; } }       // カメラステータスを表示するピクチャーボックス

        public int selectedCamera { get { return selCam; } set { selCam = value; } }                                                    // 選択されているカメラ ////////////////////////////////

        public PresetData_with_Name[] preset_data { get { return presets; } set { presets = value; } }                                  // プリセットデータ ////////////////////////////////////
        public int num_of_preset_data { get { return num_of_preset; } set { num_of_preset = value; } }                                  // プリセットの数 //////////////////////////////////////

        public bool isAnyConnection { get { return isAnyConnectioned(); } }                                                             // 受信状態のカメラが存在するか /////////////////////////
        public bool isControllerRunning { get { return isContRun; } set { isContRun = value; } }                                        // コントローラーの受信スレッドが動作しているか
        public Microsoft.DirectX.DirectInput.Device joyStick { get { return joystick; } set { joystick = value; } }                     // ジョイスティック
        public int selectedGamePad { get { return selected_GP; } set { selected_GP = value; } }                                         //選択されている(使用する)ゲームパッドの番号
        public pads padKeys { get { return padKey; } set { padKey = value; } }                                                          // パッドのキー割り当て
        ///////////////////////////////////////////////////////

        ///// ボタン連打をブロックする処理 //////////////////////////////////////////////////////////////////
        private bool buttonProcessing;
        private void Application_Idle(object sender, System.EventArgs e) { this.buttonProcessing = false; }
        ///////////////////////////////////////////////////////////////////////////////////////////////////


        // コンストラクタの処理 //
        public MainForm()
        {
            InitializeComponent();
            System.Windows.Forms.Application.Idle += new EventHandler(Application_Idle);    // ボタン連打をブロックする処理
            ServicePointManager.DefaultConnectionLimit = constClass.MAX_HTTP_CONNECTION;    // HTTPの同時接続可能数を増やす
        }


        // プログラム開始時の処理 //
        private void Form1_Load(object sender, EventArgs e)
        {
            MainForm.Form1Instance = this;  // MainForm以外からMainForm内の変数をアクセスできるようにする
            hwnd = this.Handle;     //ウインドウハンドルの取得

            initStruct();           //配列の初期化 
            loadDefaultSetting();   // 初期設定を読み出す

            // プリセットの数を初期化する //
            num_of_preset = 1;
            presets = new PresetData_with_Name[constClass.PRESET_MAX];
            presetList.SelectedIndex = 0;

            ////////// 設定をxmlファイルから読み込み //////////
            xmlClass xC = new xmlClass();
            xC.readSetting(pathes.settingFilePath);     // 設定ファイルの読み込み
            xC.readPreset(pathes.presetFilePath);       // プリセットファイルの読み込み
            xC = null;
            /////////////////////////////////////////////////

            ///////////// スクリーンの初期化 /////////////////
            jpgClass jpgC = new jpgClass();
            jpgC.muteVideoScreen(jpgClass.BLACK);           // 画面の初期化(黒くぬりつぶす)
            jpgC.muteVideoScreen(jpgClass.BLACK, true, -1); // サムネイル画面の初期化(黒くぬりつぶす)
            jpgC = null;
            /////////////////////////////////////////////////

            ptzC = new ptzClass();

            updateCamList();                                            // カメラの一覧を取得する
            selectedCamera = checkSelectedCamera();                     // 選択されているカメラの番号を取得する
            loadSettings(selectedCamera);                               // 値をフォーム上に呼び出す
            buttonStateChange(receiveFlags[selectedCamera]);            // ボタンの有効・無効を設定する            
        }



        // 受信開始(接続開始)のボタンを押したときの処理
        private void button1_Click(object sender, System.EventArgs e)
        {
            httpClass htC = new httpClass();
            JSON_Commands jsonC = new JSON_Commands();

            if (this.buttonProcessing == true)
            {
                return;
            }
            this.buttonProcessing = true;

            //受信開始・停止の判定//
            if (receiveFlags[selectedCamera] == true){   // 現在受信中の場合

                receiveFlags[selectedCamera] = false;            // 受信開始フラグをクリアする
                buttonStateChange(receiveFlags[selectedCamera]); // ボタンの入力可否を切り替える

                if (isAnyConnection == false & joyStick != null)   // すべての接続が切断され、パッドが存在する場合は一旦開放する
                {
                    joyStick.Dispose();
                    joyStick = null;
                }

                try {                           // セッション終了をカメラに通知する
                    htC.endSession();
                } catch {    
                
                }
                return;                         // 受信終了なのでこれ以上は何もせずに終了する

            }else{ //受信中で無い場合

                if (joyStick == null)   // パッドを未取得の場合は取得する
                {
                    // ジョイパッドの状態を取得する
                    joyPadClass joypadC = new joyPadClass();
                    // joyStick = joypadC.getFirstJoyPad();        // OSが認識している最初のゲームパッドを取得する
                    joystick = joypadC.getJoyPad(padKeys.gamePad, this.handler);
                    num_of_gamepad = joypadC.getNumber();

                    // デバイスに対するアクセス権をとる
                    if (joyStick != null)
                    {
                        joyStick.Acquire();
                    }
                }

                saveSettings(selectedCamera);                     // 現在の値を配列に保存する
                receiveFlags[selectedCamera] = true;              // 受信開始フラグをセットする

                ///// ここから受信開始のシーケンス /////

                // セッション取得
                try
                {
                    cookies[selectedCamera] = htC.getSession();
                }
                catch (WebException wex)
                {
                    dispSessionErrorMessage(wex);
                    return;
                }

                // カメラの機能を取得            
                camFeatures[selectedCamera] = htC.getCamFeature_Data();

                buttonStateChange(receiveFlags[selectedCamera]);　// ボタンの入力可否を切り替える

                // カメラの設定(明るさ補正など)を取得
                camAutoStatuses[selectedCamera] = htC.getCamAutoStatus();


                thStatus = new Thread(delegate()    //カメラ状態取得スレッド
                {
                    htC.getCameraStatus();
                });
                thStatus.Start();

                thSession = new Thread(delegate()   // セッション更新スレッド
                {
                    htC.updateSession();
                });
                thSession.Start();

                jpgClass jpgC = new jpgClass();
                thVideo = new Thread(delegate()     // 映像表示スレッド
                {
                    jpgC.getMjpeg();
                });
                thVideo.Start();

                thThumb = new Thread(delegate()     // サムネイル更新スレッド
                {
                    jpgC.updateThumbnail();
                });
                thThumb.Start();

                wavClass wavC = new wavClass();
                thAudio = new Thread(delegate()     // 音声再生スレッド
                {
                    wavC.getLpcm();
                });
                thAudio.Start();

                if (num_of_gamepad > 0)            // コントローラー取得スレッド(ゲームパッドの認識数が0のときは実行しない)
                {
                    joyPadClass joyPadC = new joyPadClass();
                    if (isControllerRunning == false)   // スレッドを重複起動しないように、フラグで管理する(このフラグはスレッドが終了時にfalseに戻る)
                    {
                        isControllerRunning = true;
                        thControl = new Thread(delegate()
                        {
                            joyPadC.getControl();
                        });
                        thControl.Start();
                    }
                }

            }


        }


        // プログラム終了時の処理
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (isAnyConnection == true)  // とりあえずカメラが１台でも接続している場合は終了できないようにする（馬鹿よけ）
            {
                DialogResult res = MessageBox.Show(constClass.MESSAGE_CONFIRM_QUIT, constClass.MESSAGE_CHECK, MessageBoxButtons.OKCancel);
                if ( res == DialogResult.Cancel ){
                    e.Cancel = true;
                    return;
                }else{
                    e.Cancel = false;
                    httpClass htC = new httpClass();
                    for (int i = 0; i < constClass.MAX_NUM_OF_CAMERA; i++)
                    {
                        if (receiveFlags[i] == true) {  // 受信中のカメラのセッションを閉じる
                            htC.endSession(i);
                        }
                        receiveFlags[i] = false;
                    }
                    htC = null;
                }
            }

            // 設定をXMLファイルに保存
            saveSettings(selectedCamera);   // 現在のフォームに入力されている値を変数に保存する
            xmlClass xC = new xmlClass();
            xC.writeSetting(pathes.settingFilePath);    // 設定の保存
            xC.writePreset(pathes.presetFilePath);      // プリセットの保存
            xC = null;
            this.Dispose();
        }


        // セッション取得時にエラーが発生した場合のメッセージ表示
        private void dispSessionErrorMessage(WebException wex)
        {
            if (wex.Status == WebExceptionStatus.ProtocolError)
            {
                HttpWebResponse err = (HttpWebResponse)wex.Response;

                switch (err.StatusCode)
                {
                    case HttpStatusCode.Unauthorized:   // 401(認証失敗)の場合
                        MessageBox.Show(constClass.MESSAGE_AUTH_ERROR);
                        break;
                    case HttpStatusCode.ServiceUnavailable: // 503 (サーバーエラー)の場合
                        MessageBox.Show(constClass.MESSAGE_CHECK_CAMERA_STATUS);
                        break;
                    case HttpStatusCode.NotFound:
                        MessageBox.Show(constClass.MESSAGE_CHECK_CAMERA_STATUS);
                        break;
                    default:
                        Debug.WriteLine(err.StatusCode);
                        break;
                }
                receiveFlags[selectedCamera] = false;
                buttonStateChange(receiveFlags[selectedCamera]);
                return;
            } else {
                if (wex.Status == WebExceptionStatus.Timeout)
                {
                    MessageBox.Show(constClass.MESSAGE_CHECK_CAMERA_STATUS);
                    receiveFlags[selectedCamera] = false;
                    buttonStateChange(receiveFlags[selectedCamera]);
                    return;
                } else {
                    MessageBox.Show("セッション取得でエラー: " + wex);
                    receiveFlags[selectedCamera] = false;
                    buttonStateChange(receiveFlags[selectedCamera]);
                    return;
                }
            }
        }


        // 設定画面にログインするボタンが押されたときの動作
        private void button1_Click_1(object sender, EventArgs e)
        {
            settingWindow sW = new settingWindow();
            sW.ShowDialog();
        }


        // パッド設定ボタンが押されたときの動作
        private void button2_Click_1(object sender, EventArgs e)
        {
            padSetting padS = new padSetting();
            padS.ShowDialog();
        }


        // 画面キャプチャボタンをクリックしたときの動作
        private void button1_Click_2(object sender, EventArgs e)
        {
            if (this.buttonProcessing == true) { return; }

            if (this.buttonProcessing == false & receiveFlags[selectedCamera] == true)  // 受信中でかつ、ボタンが２度押しされていない場合のみ処理する
            {
                this.buttonProcessing = true;
                jpgClass jpgC = new jpgClass();
                jpgC.saveJPG();                     // 現在の表示をJpegとして保存
                jpgC = null;
            }
        }


        // 履歴画面表示ボタンが押されたときの動作
        private void button3_Click(object sender, EventArgs e)
        {
            HistoryForm hF = new HistoryForm();
            hF.ShowDialog(); //履歴画面表示中は裏のフォームを触れないようにする
        }


        // カメラ一覧取得ボタンが押されたときの動作
        private void button4_Click(object sender, EventArgs e)
        {
            if (this.buttonProcessing == true){ return; }
            this.buttonProcessing = true;

            button_CamSearch.Enabled = false;
            updateCamList();
            button_CamSearch.Enabled = true;
        }


        // カメラの一覧を取得する関数
        private void updateCamList() 
        {
            camSearchClass cSC = new camSearchClass();
            cameraList.Items.Clear();
            string[] res = cSC.searchCamera();

            if (res == null) {
                return ; 
            }
            for (int i = 0; i < res.Count(); i++) {
                cameraList.Items.Add(res[i]); 
            }
        }


        // CamList上の一覧を選択（クリック）したときの動作
        private void cameraList_Click(object sender, EventArgs e)
        {
            if (cameraList.SelectedItems.Count == 1)
            {
                DialogResult result = MessageBox.Show(constClass.MESSAGE_CONFIRM_REGIST, constClass.MESSAGE_CHECK, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2);
                if (result == DialogResult.Yes)
                {
                    ipAddr = cameraList.Text.Substring(0,cameraList.Text.LastIndexOf(":"));
                    portNumber = cameraList.Text.Substring(cameraList.Text.LastIndexOf(":"));
                }
            }
        }


        // パンチルトボタン
        // センターボタン
        private void button2_Click(object sender, EventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(constClass.CMD_PAN_TILT_ABSOLUTE, 0, 0, false);
            }
        }
        
        // 左ボタン
        // 押したとき
        private void button_LEFT_MouseDown(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(-2 * pantilt_Speed -1, 0, 0, cameraSettings[selectedCamera].inversePicture);
            }
        }
        // 離したとき
        private void button_LEFT_MouseUp(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(constClass.CMD_PAN_STOP, 0, 0, false);
            }
        }
        
        // 右ボタン
        // 押したとき
        private void button_RIGHT_MouseDown(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(2* pantilt_Speed + 1, 0, 0, cameraSettings[selectedCamera].inversePicture);
            }
        }
        // 離したとき
        private void button_RIGHT_MouseUp(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(constClass.CMD_PAN_STOP, 0, 0, false);
            }
        }
        
        // 上ボタン
        // 押したとき
        private void button_UP_MouseDown(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(constClass.CMD_TILT_UP_MIN + (int)pantiltSpeed.Value, 0, 0, inversePicture);
            }
        }
        // 離したとき
        private void button_UP_MouseUp(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(constClass.CMD_TILT_STOP, 0, 0, false);
            }
        }
        
        // 下ボタン
        // 押したとき
        private void button_DOWN_MouseDown(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(constClass.CMD_TILT_DOWN_MIN - (int)pantiltSpeed.Value, 0, 0, inversePicture);
            }
        }
        // 離したとき
        private void button_DOWN_MouseUp(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postPTCommand(constClass.CMD_TILT_STOP2, 0, 0, false);
            }
        }

        // ズームボタン
        // Tズームボタン
        // 押したとき
        private void button_Tzoom_MouseDown(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postZoomCommand((int)(zoomSpeed.Value));
            }
        }
        // 離したとき
        private void button_Tzoom_MouseUp(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postZoomCommand(0);
            }
        }

        // Wズームボタン
        // 押したとき
        private void button_Wzoom_MouseDown(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postZoomCommand((int)(zoomSpeed.Value)*-1);
            }
        }
        // 離したとき
        private void button_Wzoom_MouseUp(object sender, MouseEventArgs e)
        {
            if (camFeatures[selectedCamera].HasMechaPanTilt == true)    // メカパンチルトがあるとき(LS2のとき)
            {
                ptzC.postZoomCommand(0);
            }
        }

        // フォーカス(近)ボタン
        // 押したとき
        private void button2_MouseDown(object sender, MouseEventArgs e)
        {
            if (camStatuses[selectedCamera].IsFocusAuto == false)
            {
                camControl cc = new camControl();
                cc.postFOCUS_NEAR();
            }
        }
        // 離したとき
        private void button2_MouseUp(object sender, MouseEventArgs e)
        {
            if (camStatuses[selectedCamera].IsFocusAuto == false)
            {
                camControl cc = new camControl();
                cc.postFOCUS_NEAR_STOP();
            }
        }

        // フォーカス(遠)ボタン
        // 押したとき
        private void button3_MouseDown(object sender, MouseEventArgs e)
        {
            if (camStatuses[selectedCamera].IsFocusAuto == false)
            {
                camControl cc = new camControl();
                cc.postFOCUS_FAR();
            }
        }
        // 離したとき
        private void button3_MouseUp(object sender, MouseEventArgs e)
        {
            if (camStatuses[selectedCamera].IsFocusAuto == false)
            {
                camControl cc = new camControl();
                cc.postFOCUS_FAR_STOP();
            }
        }

        // フォーカスA/M切替ボタン押したとき
        private void button4_Click_1(object sender, EventArgs e)
        {
            camControl cc = new camControl();

            if (camStatuses[selectedCamera].IsFocusAuto == true)
            {
                cc.postFOCUS_MANUAL();
            }
            else
            {
                cc.postFOCUS_AUTO();
            }

        }

        // REC STARTボタン押したとき
        private void button_REC_START_Click(object sender, EventArgs e)
        {
            if (camFeatures[selectedCamera].Mpeg2tsSize == "OFF") {
                camControl cc = new camControl();
                cc.postREC_START();
            }
        }

        // REC STOPボタン押したとき
        private void button5_Click(object sender, EventArgs e)
        {
            if (camFeatures[selectedCamera].Mpeg2tsSize == "OFF")
            {
                camControl cc = new camControl();
                cc.postREC_STOP();
            }
        }


        // プリセットの保存・復元処理 //
        private void button_Preset_Save_Click(object sender, EventArgs e)
        {
            presetClass presetC = new presetClass();
            PresetData_without_Name ptz_pos = new PresetData_without_Name();
            httpClass htC = new httpClass();

            ptz_pos = htC.GetPTZ();                                                                                   // カメラから現在のパンチルト、ズームの状態を取得
            double zoom = (double)ptz_pos.DeciZoomPosition / 10.0;                                                      // ズーム倍率を取得
            string presetName = "PAN: " + ptz_pos.PanPosition + ", TILT: " + ptz_pos.TiltPosition + ", ZOOM: " + zoom;  // プリセット名を生成
            
            if (presetList.SelectedIndex == -1) // リストが選ばれてない場合は何もしない
            {
                return;
            }

            if (presetList.SelectedIndex == 0) { // 一番頭のリストが選ばれている場合は新規にプリセットを追加する
                if (num_of_preset < constClass.PRESET_MAX)
                {
                    presetList.Items.Add(presetName);
                    num_of_preset += 1;
                    preset_data[num_of_preset] = presetC.returnPresetData_with_Name(ptz_pos.PanTiltValid, ptz_pos.PanPosition, ptz_pos.TiltPosition, ptz_pos.DeciZoomPosition,presetName);
                    
                } else {
                    preset_data[presetList.SelectedIndex] = presetC.returnPresetData_with_Name(ptz_pos.PanTiltValid, ptz_pos.PanPosition, ptz_pos.TiltPosition, ptz_pos.DeciZoomPosition, presetName);
                }
            };

            return;
        }


        // ダブルクリック時、選択されたプリセットの値を復元する
        private void listBox_Preset_DoubleClick(object sender, EventArgs e)
        {
            if (presetList.SelectedIndex > 0 & receiveFlags[selectedCamera] == true)
            {
                ptzC.postPTCommand(constClass.CMD_PAN_TILT_ABSOLUTE, presets[presetList.SelectedIndex].PanPosition, presets[presetList.SelectedIndex].TiltPosition, false);
                ptzC.postZoomPositionCommand(presets[presetList.SelectedIndex].DeciZoomPosition, 3);
            }
        }


        // 選択されたプリセットを削除する
        private void button_Preset_Delete_Click(object sender, EventArgs e)
        {
            presetClass presetC = new presetClass();
            presetC.deletePreset(presetList.SelectedIndex);
        }


        // 選択されたプリセットを編集する
        private void button_Preset_Edit_Click(object sender, EventArgs e)
        {
            if (presetList.SelectedIndex > 0)
            {
                presetEdit pE = new presetEdit();
                pE.ShowDialog();
            }
        }


        // 選択されているカメラ(の番号)を返す
        private int checkSelectedCamera() { 
            if (radioButton_cam1.Checked == true){ return 0; }
            if (radioButton_cam2.Checked == true){ return 1; }
            if (radioButton_cam3.Checked == true){ return 2; }
            if (radioButton_cam4.Checked == true){ return 3; }
            return 0;
        }


        // 現在いずれかのカメラに接続されているかどうかを返す
        private bool isAnyConnectioned() {
            for (int i = 0; i < constClass.MAX_NUM_OF_CAMERA; i++) {
                if (receiveFlags[i] == true) { return true; }
            }
            return false;
        }


        // 各構造体・配列の初期化
        private void initStruct() {
            recFlags = new bool[constClass.MAX_NUM_OF_CAMERA];                  // 各カメラの受信状態フラグ
            camSettings = new cameraSetting[constClass.MAX_NUM_OF_CAMERA];      // 各カメラの設定
            camFeatures = new Cam_Feature_Data[constClass.MAX_NUM_OF_CAMERA];   // 各カメラの機能
            camStatuses = new Cam_Status[constClass.MAX_NUM_OF_CAMERA];         // カメラの状態
            camAutoStatuses = new Cam_AutoStatus[constClass.MAX_NUM_OF_CAMERA];  // カメラの状態(明るさ補正など)
            padKey = new pads();                                                // パッドのキー割り当て
            path_ = new pathS();                                                // 各種パス(ディレクトリ)
        }


        // 各種パスを取得する
        private pathS getPathes() {
            pathS result = new pathS();
            result.applicationPath = Application.ExecutablePath;
            result.applicationPath = result.applicationPath.Substring(0, result.applicationPath.LastIndexOf(@"\") + 1); // アプリケーション実行パス(ディレクトリ)
            result.settingFilePath = result.applicationPath + "setting_receive.xml";                                    // 設定ファイルのパス
            result.presetFilePath = result.applicationPath + "preset.xml";                                              // プリセットファイルのパス
            return result;        
        }


        // 各設定の初期値を呼び出す
        private void loadDefaultSetting()
        {
            pads pK;
            cookies = new CookieContainer[constClass.MAX_NUM_OF_CAMERA];        // クッキー
            selectedCamera = constClass.DEFAULT_SELECTED_CAMERA;                // 選択されているカメラ（の番号)
            path_ = getPathes();                                                // 各種パス(ディレクトリ)

            pK.gamePad = joyPadClass.PAD_DEFAULT;                               // パッド設定
            pK.PanTilt_Up = joyPadClass.PAD_DEFAULT_PANTILT_SPEED_UP;
            pK.PanTilt_Down = joyPadClass.PAD_DEFAULT_PANTILT_SPEED_DOWN;
            pK.T_Zoom = joyPadClass.PAD_DEFAULT_Z_ZOOM;
            pK.W_Zoom = joyPadClass.PAD_DEFAULT_W_ZOOM;
            pK.Center_Reset = joyPadClass.PAD_DEFAULT_CENTER_RESET;
            padKeys = pK;

            for (int i = 0; i < constClass.MAX_NUM_OF_CAMERA; i++)
            {
                cameraSettings[i].userName = constClass.DEFAULT_USERNAME;                     //各カメラのユーザー名
                cameraSettings[i].passWord = constClass.DEFAULT_PASSWORD;                     //各カメラのパスワード
                cameraSettings[i].ipAddress = constClass.DEFAULT_IPADDRESS;                   //各カメラのIPアドレス
                cameraSettings[i].portNumber = constClass.DEFAULT_PORT;                       //各カメラのポート番号
                cameraSettings[i].useProxy = constClass.DEFAULT_USE_PROXY;                    //各カメラのプロキシ使用フラグ
                cameraSettings[i].inversePicture = constClass.DEFAULT_INVERSE_PICTURE;        //各カメラの画像反転フラグ
                cameraSettings[i].playSound = constClass.DEFAULT_PLAY_SOUND;                  //各カメラの音声出力フラグ
                cameraSettings[i].overlayString = constClass.DEFAULT_OVERLAY_STRING;          //各カメラの文字重畳フラグ
                cameraSettings[i].overlayStringData = constClass.DEFAULT_OVERLAY_STRING_DATA; //各カメラに重畳する文字
                receiveFlags[i] = constClass.DEFAULT_RECEIVE_FLAG;              //各カメラのデータ受信フラグ
                cookies[i] = new CookieContainer();                             //各カメラのクッキー
            };          
        }


        // 各カメラの設定値<-->フォーム間のデータ読み出し、格納
        // 設定値->フォームへ読み出し
        private void loadSettings(int cam)
        {
            userName = camSettings[cam].userName;
            passWord = camSettings[cam].passWord;
            ipAddr = camSettings[cam].ipAddress;
            portNumber = camSettings[cam].portNumber;
            useProxy = camSettings[cam].useProxy;
            inversePicture = camSettings[cam].inversePicture;
            playSound = camSettings[cam].playSound;
            overlayString = camSettings[cam].overlayString;
            overlayStringData = camSettings[cam].overlayStringData;
        }
        // フォーム->設定値へ格納
        private void saveSettings(int cam)
        {
            camSettings[cam].userName = userName;
            camSettings[cam].passWord = passWord;
            camSettings[cam].ipAddress = ipAddr; ;
            camSettings[cam].portNumber = portNumber;
            camSettings[cam].useProxy = useProxy;
            camSettings[cam].inversePicture = inversePicture;
            camSettings[cam].playSound = playSound;
            camSettings[cam].overlayString = overlayString;
            camSettings[cam].overlayStringData = overlayStringData;
        }


        // ボタンの有効・無効状態を切り替える
        private void buttonStateChange(bool receiving)
        {
            button_Preset_Save.Enabled = receiving;     // プリセット保存ボタンの操作可否
            testBox_IP.Enabled = !receiving;            // IPアドレスの入力可否
            port.Enabled = !receiving;                  // ポート番号の入力可否
            cameraList.Enabled = !receiving;            // カメラ一覧の変更可否
            button_CamSearch.Enabled = !receiving;      // カメラ検索ボタン
            textBox_USER.Enabled = !receiving;          // ユーザー名の入力可否
            textBox_PASS.Enabled = !receiving;          // パスワードの入力可否
            checkBox_useProxy.Enabled = !receiving;     // プロキシを使用するか
            checkBox_playSound.Enabled = !receiving;    // 音声を出力するか

            button_Setting.Enabled = !receiving;        // 設定画面呼び出しボタンの入力可否
            button_UP.Enabled = receiving;              // パンチルト・ズームボタンの入力可否
            button_DOWN.Enabled = receiving;
            button_LEFT.Enabled = receiving;
            button_RIGHT.Enabled = receiving;
            button_Tzoom.Enabled = receiving;
            button_Wzoom.Enabled = receiving;
            button_Center.Enabled = receiving;

            button_FOCUS_AM.Enabled = receiving;    // フォーカスボタン
            button_FOCUS_FAR.Enabled = receiving;
            button_FOCUS_NEAR.Enabled = receiving;
            button_REC_START.Enabled = receiving & camFeatures[selectedCamera].Mpeg2tsSize == "OFF";   // RECボタン
            button_REC_STOP.Enabled = receiving & camFeatures[selectedCamera].Mpeg2tsSize == "OFF";

            button_PadSetting.Enabled = !isAnyConnection;   // パッド設定変更ボタン

            button_AutoSetting.Enabled = receiving; // カメラ設定ボタン

            if (receiving)
            {
                button_Start.Text = "受信停止";   //
            }
            else
            {
                button_Start.Text = "受信";
            }

            return;
        }


        // ラジオボタンの状態が変化したときの処理
        private void radioButton_cam1_CheckedChanged(object sender, EventArgs e) { if (radioButton_cam1.Checked == true) { radioButton_Changed(); } }
        private void radioButton_cam2_CheckedChanged(object sender, EventArgs e) { if (radioButton_cam2.Checked == true) { radioButton_Changed(); } }
        private void radioButton_cam3_CheckedChanged(object sender, EventArgs e) { if (radioButton_cam3.Checked == true) { radioButton_Changed(); } }
        private void radioButton_cam4_CheckedChanged(object sender, EventArgs e) { if (radioButton_cam4.Checked == true) { radioButton_Changed(); } }

        private void radioButton_Changed() {
            saveSettings(selectedCamera);                       // フォーム上の情報を変数に保存する
            selectedCamera = checkSelectedCamera();             // 選択されているカメラの番号を取得する
            loadSettings(selectedCamera);                       // 選択されているカメラの設定をフォームに反映する
            buttonStateChange(receiveFlags[selectedCamera]);    // ボタンの有効・無効を設定する

            if (receiveFlags[selectedCamera] == false)          // 選択されたカメラが映像受信状態で無い場合は、画面を黒画に変更する
            {
                jpgClass jpgC = new jpgClass();
                jpgC.muteVideoScreen(jpgClass.BLACK);
            }
        }


        // 画面反転のチェックボックスが変更されたときに、変数に反映する
        private void checkBox_InversePicture_CheckedChanged(object sender, EventArgs e)
        {
            cameraSettings[selectedCamera].inversePicture = checkBox_InversePicture.Checked;
        }


        // 音声出力のフラグが変更されたときに、変数に反映する
        private void checkBox_playSound_CheckedChanged(object sender, EventArgs e)
        {
            cameraSettings[selectedCamera].playSound = checkBox_playSound.Checked;
        }

        
        // 文字重畳のチェックボックスが変更されたときに、変数に反映する
        private void checkBox_overlayText_CheckedChanged(object sender, EventArgs e)
        {
            cameraSettings[selectedCamera].overlayString = checkBox_overlayText.Checked;
        }


        // 重畳する文字列が変更されたときに、変数に反映する
        private void textBox_overlayText_TextChanged(object sender, EventArgs e)
        {
            cameraSettings[selectedCamera].overlayStringData = textBox_overlayText.Text;
        }

        private void button_AutoSetting_Click(object sender, EventArgs e)
        {
            autoSetting aS = new autoSetting();
            aS.StartPosition = FormStartPosition.Manual;
            aS.Location = new Point(MainForm.ActiveForm.Left + MainForm.ActiveForm.Width,MainForm.ActiveForm.Top);
            aS.ShowDialog();
        }



    }
}
