﻿// 構造体の宣言

namespace WindowsFormsApplication1
{
    // 実行パスや設定ファイルのパスを格納する構造体
    public struct pathS {
        public string applicationPath { get; set; }     // アプリケーションの実行パス
        public string settingFilePath { get; set; }     // 設定ファイルのパス
        public string presetFilePath { get; set; }      // プリセットファイルのパス
    }


    // 各カメラの設定値を格納する構造体
    public struct cameraSetting
    {
        public string ipAddress { get; set; }           // IPアドレス
        public string portNumber { get; set; }          // ポート番号
        public string userName { get; set; }            // ユーザー名(管理者)
        public string passWord { get; set; }            // パスワード(管理者)
        public bool useProxy { get; set; }              // プロキシを使用するかどうか
        public bool playSound { get; set; }             // 音声を再生するかどうか
        public bool inversePicture { get; set; }        // 画面を反転するかどうか
        public bool overlayString { get; set; }         // 文字を重畳するかどうか
        public string overlayStringData { get; set; }   // 文字を重畳する場合の文字列データ
    }


    // カメラの機能情報を格納する構造体
    public struct Cam_Feature_Data
    {
        public int CamVer { get; set; }
        public string IsAdmin { get; set; }
        public bool HasMechaPanTilt { get; set; }
        public int MechaPanMax { get; set; }
        public int MechaPanMin { get; set; }
        public int MechaTiltMax { get; set; }
        public int MechaTiltMin { get; set; }
        public int MechaSpeedMax { get; set; }
        public int MechaSpeedMin { get; set; }
        public int DeciZoomMax { get; set; }
        public int DeciZoomMin { get; set; }
        public bool HasDegitalPanTilt { get; set; }
        public int DegitalXMax { get; set; }
        public int DegitalXMin { get; set; }
        public int DegitalYMax { get; set; }
        public int DegitalYMin { get; set; }
        public int DigitalDeciZoomMax { get; set; }
        public int DigitalDeciZoomMin { get; set; }
        public int DigitalPTZSpeedMax { get; set; }
        public int DigitalPTZSpeedMin { get; set; }
        public string Mpeg2tsSize { get; set; }
        public string FrameRate { get; set; }
        public string VideoSignalType { get; set; }
    }


    // カメラの状態を格納する構造体
    public struct Cam_Status
    {
        public int BatteryLevel { get; set; }           // バッテリーレベル
        public bool AcExist { get; set; }               // ACアダプタ接続
        public string SdcardExist { get; set; }         // SDカードの有無
        public string SdcardRemains { get; set; }       // SDカードの残量
        public string CamMotion { get; set; }           // ?
        public bool IsFocusAuto { get; set; }           // オート・マニュアルフォーカス
        public bool HasMotionJpegRes { get; set; }      // ?
        public bool HasLpcmRes { get; set; }            // ?
        public bool HasMpeg2tsRes { get; set; }         // ?
        public string CamMode { get; set; }             // ?
        public string CamIndexVideoFormat { get; set; } // ?
        public int ZoomLevel { get; set; }              // ズーム（なんだけど意味不明な値が出る)
        public bool IsSleepImageSensor { get; set; }    // センサーがスリープしているか
    }

    // カメラの状態(明るさ補正など)を格納する構造代
    public struct Cam_AutoStatus
    {
        public string DIS { get; set; }                 // 手ぶれ補正(DIS)
        public string BrightnessLevel { get; set; }     // 明るさ補正
        public string ShutterSpeed { get; set; }        // シャッタースピード
        public string WhiteBalance { get; set; }        // ホワイトバランス
        public string VideoQuality { get; set; }        // 動画画質
        public string DateRecPosition { get; set; }     // 日時記録
        public string IsDateRecSec { get; set; }        // 秒の表示
        public string Tally { get; set; }               // タリー(ランプ?)
        public string Light { get; set; }               // ライト
        public string Focus { get; set; }               // フォーカス
        public string FrameInRec { get; set; }          // フレームインレック
        public string GainUp { get; set; }              // 感度?
    }


    // プリセットデータを格納する構造体(名前付き。プリセットの値の管理に使う)
    public struct PresetData_with_Name
    {
        public bool PanTiltValid { get; set; }
        public int PanPosition { get; set; }
        public int TiltPosition { get; set; }
        public int DeciZoomPosition { get; set; }
        public string Name { get; set; }
    }


    // プリセットデータを格納する構造体(名前なし。カメラから値を拾うときに使う)
    public struct PresetData_without_Name
    {
        public bool PanTiltValid { get; set; }
        public int PanPosition { get; set; }
        public int TiltPosition { get; set; }
        public int DeciZoomPosition { get; set; }
    }


    // パッドのキー割り当てを保存する構造体
    public struct pads           
    {
        public int gamePad;         // パッドの番号
        public int PanTilt_Up;      // パンチルト加速
        public int PanTilt_Down;    // パンチルト減速
        public int T_Zoom;          // Tズーム
        public int W_Zoom;          // Wズーム
        public int Center_Reset;    // センターリセット
    }




}
