﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WindowsFormsApplication1
{
    public partial class autoSetting : Form
    {
        public const string DIS = "DIS";
        public const string DIS_OFF = "OFF";
        public const string DIS_WEAK = "SOFT";
        public const string DIS_MEDIUM = "MEDIUM";
        public const string DIS_STRONG = "STRONG";
        
        public const string BRIGHT = "BrightnessLevel";
        public const string BRIGHT_AUTO = "AUTO";
        
        public const string SHUTTER = "ShutterSpeed";
        public const string SHUTTER_AUTO = "AUTO";

        public const string WB = "WhiteBalance";
        public const string WB_AUTO = "AUTO";
        public const string WB_FINE = "FINE";
        public const string WB_CLOUD = "CLOUD";
        public const string WB_HALOGEN = "HALOGEN";
        public const string WB_ONETOUCH = "MWB";

        public const string DATE = "DateRecPosition";
        public const string DATE_OFF = "OFF";
        public const string DATE_UL = "UPPER_LEFT";
        public const string DATE_UR = "UPPER_RIGHT";
        public const string DATE_LL = "LOWER_LEFT";
        public const string DATE_LR = "LOWER_RIGHT";

        public const string SECOND = "IsDateRecSec";
        public const string SECOND_OFF = "OFF";
        public const string SECOND_ON = "ON";

        public const string LIGHT = "Light";
        public const string LIGHT_AUTO = "AUTO";
        public const string LIGHT_OFF = "OFF";
        public const string LIGHT_ON = "ON";


        public bool LoadAutomatic = false;  // アプリが勝手にフォームをいじる場合はTrueにしておく
                                                // (ユーザーの操作によるフォームの状態変更と区別できるようにするため)
        public autoSetting()
        {
            InitializeComponent();
        }

        private void autoSetting_Load(object sender, EventArgs e)
        {
            Cam_AutoStatus cas = new Cam_AutoStatus();
            httpClass htC = new httpClass();

            cas = htC.getCamAutoStatus();

            LoadAutomatic = true;

            // カメラから各種設定を読み取る
            readDIS(cas.DIS);                   // 手ぶれ補正
            readBRIGHT(cas.BrightnessLevel);    // 明るさ補正
            readSHUTTER(cas.ShutterSpeed);      // シャッタースピード
            readWB(cas.WhiteBalance);           // ホワイトバランス
            readDATE(cas.DateRecPosition);      // 日時記録
            readSECOND(cas.IsDateRecSec);       // 秒の表示
            readLIGHT(cas.Light);               // ライト

            LoadAutomatic = false;

        }



        // カメラから各種設定を読み取る関数
        // 手ぶれ補正
        private void readDIS(string DIS_Setting) 
        {
            switch (DIS_Setting)
            {
                case DIS_OFF:
                    radioButton_DIS_OFF.Checked = true;
                    break;
                case DIS_WEAK:
                    radioButton_DIS_WEAK.Checked = true;
                    break;
                case DIS_MEDIUM:
                    radioButton_DIS_MEDIUM.Checked = true;
                    break;
                case DIS_STRONG:
                    radioButton_DIS_STRONG.Checked = true;
                    break;
            }
        }

        // 明るさ補正
        private void readBRIGHT(string BRIGHT_Setting) 
        {
            if (BRIGHT_Setting == BRIGHT_AUTO)
            {
                radioButton_BRIGHT_AUTO.Checked = true;
                comboBox_BRIGHT.Text = "0";
            }
            else
            {
                radioButton_BRIGHT_MANUAL.Checked = true;
                comboBox_BRIGHT.Text = BRIGHT_Setting;
            }
        }

        // シャッタースピード
        private void readSHUTTER(string SHUTTER_Setting) 
        {
            if (SHUTTER_Setting == SHUTTER_AUTO)
            {
                radioButton_SHUTTER_AUTO.Checked = true;
                comboBox_SHUTTER.Text = "1/60";
            }
            else
            {
                radioButton_SHUTTER_MANUAL.Checked = true;
                comboBox_SHUTTER.Text = SHUTTER_Setting;
            }
        }

        // ホワイトバランス
        private void readWB(string WB_Setting) 
        {
            switch (WB_Setting)
            {
                case WB_AUTO:
                    comboBox_WB.Text = "オート";
                    break;
                case WB_FINE:
                    comboBox_WB.Text = "はれ";
                    break;
                case WB_CLOUD:
                    comboBox_WB.Text = "くもり";
                    break;
                case WB_HALOGEN:
                    comboBox_WB.Text = "ハロゲン";
                    break;
                case WB_ONETOUCH:
                    comboBox_WB.Text = "ワンタッチ";
                    break;
            }
        }

        // 日時記録
        private void readDATE(string DATE_Setting) 
        {
            switch (DATE_Setting)
            {
                case DATE_OFF:
                    radioButton_DATE_OFF.Checked = true;
                    break;
                case DATE_UL:
                    radioButton_DATE_UL.Checked = true;
                    break;
                case DATE_UR:
                    radioButton_DATE_UR.Checked = true;
                    break;
                case DATE_LL:
                    radioButton_DATE_LL.Checked = true;
                    break;
                case DATE_LR:
                    radioButton_DATE_LR.Checked = true;
                    break;
            }
        }

        // 秒の表示
        private void readSECOND(string SECOND_Setting) 
        {
            if (SECOND_Setting == "ON")
            {
                checkBox_DATE_SECOND.Checked = true;
            }
            else {
                checkBox_DATE_SECOND.Checked = false;            
            }
        }

        // ライト
        private void readLIGHT(string LIGHT_Setting) 
        {
            switch (LIGHT_Setting) { 
                case LIGHT_AUTO:
                    radioButton_LIGHT_AUTO.Checked = true;
                    break;
                case LIGHT_OFF:
                    radioButton_LIGHT_OFF.Checked = true;
                    break;
                case LIGHT_ON:
                    radioButton_LIGHT_ON.Checked = true;
                    break;            
            }
        }

        ////// 手ぶれ補正の設定が変更されたときの処理
        private void radioButton_DIS_OFF_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_DIS_Changed();
            }
        }
        private void radioButton_DIS_WEAK_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_DIS_Changed();
            }
        }
        private void radioButton_DIS_MEDIUM_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_DIS_Changed();
            }
        }
        private void radioButton_DIS_STRONG_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_DIS_Changed();
            }
        }
        private void radioButton_DIS_Changed()
        {
            camControl cC = new camControl();
            if (radioButton_DIS_OFF.Checked == true) { cC.postAutoCtrlCommand(DIS, DIS_OFF); }
            if (radioButton_DIS_WEAK.Checked == true) { cC.postAutoCtrlCommand(DIS, DIS_WEAK); }
            if (radioButton_DIS_MEDIUM.Checked == true) { cC.postAutoCtrlCommand(DIS, DIS_MEDIUM); }
            if (radioButton_DIS_STRONG.Checked == true) { cC.postAutoCtrlCommand(DIS, DIS_STRONG); }
        }


        ////////////////////////////////////////////////////////////////////

        /////// 明るさ補正が変更されたときの処理
        private void radioButton_BRIGHT_AUTO_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_BIGHT_Changed();
            }
        }
        private void radioButton_BRIGHT_MANUAL_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_BIGHT_Changed();
            }
        }
        private void radioButton_BIGHT_Changed() {
            camControl cC = new camControl();
            if (radioButton_BRIGHT_AUTO.Checked == true) { cC.postAutoCtrlCommand(BRIGHT, BRIGHT_AUTO); }
            if (radioButton_BRIGHT_MANUAL.Checked == true) { cC.postAutoCtrlCommand(BRIGHT, comboBox_BRIGHT.Text); }       
        }
        private void comboBox_BRIGHT_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                comboBox_BRIGHT_Changed();        
            }
        }
        private void comboBox_BRIGHT_Changed() {
            camControl cC = new camControl();
            radioButton_BRIGHT_MANUAL.Checked = true;
            cC.postAutoCtrlCommand(BRIGHT, comboBox_BRIGHT.Text);
        }

        /////////////////////////////////////////////////////////////////

        ///// シャッタースピードが変更されたときの処理 
        private void radioButton_SHUTTER_AUTO_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_SHUTTER_Changed();
            }
        }

        private void radioButton_SHUTTER_MANUAL_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_SHUTTER_Changed();
            }
        }
        private void radioButton_SHUTTER_Changed() {
            camControl cC = new camControl();
            if (radioButton_SHUTTER_AUTO.Checked == true) { cC.postAutoCtrlCommand(SHUTTER, SHUTTER_AUTO); }
            if (radioButton_SHUTTER_MANUAL.Checked == true) { cC.postAutoCtrlCommand(SHUTTER, comboBox_SHUTTER.Text); }
        }
        private void comboBox_SHUTTER_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                comboBox_SHUTTER_Changed();
            }

        }
        private void comboBox_SHUTTER_Changed() {
            camControl cC = new camControl();
            radioButton_SHUTTER_MANUAL.Checked = true;
            cC.postAutoCtrlCommand(SHUTTER, comboBox_SHUTTER.Text);        
        }
        ////////////////////////////////////////////////////////////////////

        ////// ホワイトバランスが変更されたときの処理
        private void comboBox_WB_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false)
            {
                radioButton_WB_Changed();
            }
        }
        private void radioButton_WB_Changed() {
            camControl cC = new camControl();
            switch (comboBox_WB.Text) { 
                case "オート":
                    cC.postAutoCtrlCommand(WB, WB_AUTO);
                    break;
                case "はれ":
                    cC.postAutoCtrlCommand(WB, WB_FINE);
                    break;
                case "くもり":
                    cC.postAutoCtrlCommand(WB, WB_CLOUD);
                    break;
                case "ハロゲン":
                    cC.postAutoCtrlCommand(WB, WB_HALOGEN);
                    break;
                case "ワンタッチ":
                    cC.postAutoCtrlCommand(WB, WB_ONETOUCH);
                    break;
            }
        }
        private void button_WB_IMPORT_Click(object sender, EventArgs e) // ホワイトバランスの取り込み(未作成)
        {
            camControl cC = new camControl();
            // cc.postWBImport();
            comboBox_WB.Text = "ワンタッチ";
        }
        ///////////////////////////////////////////////////////////////////

        ////// 日時記録が変更されたときの処理
        private void radioButton_DATE_OFF_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false & radioButton_DATE_OFF.Checked == true)
            {
                radioButton_DATE_Changed();
            }
        }
        private void radioButton_DATE_UL_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false & radioButton_DATE_UL.Checked == true)
            {
                radioButton_DATE_Changed();
            }
        }
        private void radioButton_DATE_UR_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false & radioButton_DATE_UR.Checked == true)
            {
                radioButton_DATE_Changed();
            }
        }
        private void radioButton_DATE_LL_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false & radioButton_DATE_LL.Checked == true)
            {
                radioButton_DATE_Changed();
            }
        }
        private void radioButton_DATE_LR_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false & radioButton_DATE_LR.Checked == true)
            {
                radioButton_DATE_Changed();
            }
        }
        private void radioButton_DATE_Changed() {
            camControl cC = new camControl();
            if (radioButton_DATE_OFF.Checked == true) { cC.postAutoCtrlCommand(DATE, DATE_OFF); }
            if (radioButton_DATE_UL.Checked == true) { cC.postAutoCtrlCommand(DATE, DATE_UL); }
            if (radioButton_DATE_UR.Checked == true) { cC.postAutoCtrlCommand(DATE, DATE_UR); }
            if (radioButton_DATE_LL.Checked == true) { cC.postAutoCtrlCommand(DATE, DATE_LL); }
            if (radioButton_DATE_LR.Checked == true) { cC.postAutoCtrlCommand(DATE, DATE_LR); }
        }
        private void checkBox_DATE_SECOND_CheckedChanged(object sender, EventArgs e)
        {
            checkBox_DATE_SECOND_Changed();    
        }
        private void checkBox_DATE_SECOND_Changed() 
        {
            camControl cC = new camControl();
            if (checkBox_DATE_SECOND.Checked == true)
            {
                cC.postAutoCtrlCommand(SECOND, SECOND_ON);
            }
            else {
                cC.postAutoCtrlCommand(SECOND, SECOND_OFF);            
            }
        }
        /////////////////////////////////////////////////////////////////

        ////// ライトが変更されたときの処理
        private void radioButton_LIGHT_OFF_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false & radioButton_LIGHT_OFF.Checked == true)
            {
                radioButton_LIGHT_Changed();
            }
        }
        private void radioButton_LIGHT_ON_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false & radioButton_LIGHT_ON.Checked == true)
            {
                radioButton_LIGHT_Changed();
            }
        }
        private void radioButton_LIGHT_AUTO_CheckedChanged(object sender, EventArgs e)
        {
            if (LoadAutomatic == false & radioButton_LIGHT_AUTO.Checked == true)
            {
                radioButton_LIGHT_Changed();
            }
        }
        private void radioButton_LIGHT_Changed() {
            camControl cC = new camControl();
            if (radioButton_LIGHT_OFF.Checked == true) { cC.postAutoCtrlCommand(LIGHT, LIGHT_OFF); }
            if (radioButton_LIGHT_ON.Checked == true) { cC.postAutoCtrlCommand(LIGHT, LIGHT_ON); }
            if (radioButton_LIGHT_AUTO.Checked == true) { cC.postAutoCtrlCommand(LIGHT, LIGHT_AUTO); }
        }
        /////////////////////////////////////////////////////////////////////

    }
}
