﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Net.NetworkInformation;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Threading;
using System.IO;
using System.Diagnostics;

namespace WindowsFormsApplication1
{
    class camSearchClass
    {
        public const string MULTICAST_ADDRESS = "239.255.255.247";
        public const int MULTICAST_PORT = 7874;
        public const int MULTICAST_SOURCE_PORT = 50000;
        public const int MULTICAST_TIMETOLIVE = 4;
        public const int MULTICAST_TIMEOUT = 1000;

        [DataContract]
        public class sendJsonData   //送信データ
        {
            [DataMember] public string Command { get; set; }
        }

        [DataContract]
        public class receiveJsonData //受信データ
        {
            [DataMember] public string Requested { get; set; }
            [DataMember] public string Result { get; set; }
            [DataMember(Name = "Data")] public Data Data { get; set; }
        }

        [DataContract]
        public class Data //受信データ内のDataフィールド
        {
            [DataMember] public string CameraName { get; set; }
            [DataMember] public string Scheme { get; set; }
            [DataMember] public string Host { get; set; }
            [DataMember] public int Port { get; set; }
        }

        private UdpClient sendUdp;  // 送受信用のUDPクライアント

        // カメラを検索する(IPアドレスのリストを返す。外部からは通常はこちらを使う)
        public string[] searchCamera() {
            string[] result ={};
            return searchCamera(MULTICAST_ADDRESS,MULTICAST_PORT,MULTICAST_SOURCE_PORT,MULTICAST_TIMETOLIVE,MULTICAST_TIMEOUT);
        }


        // 通常外部からは使わない /////////////////////////////////////////
        // カメラを検索する(IPアドレスのリストを返す)
        private string[] searchCamera(string ipAddr_, int sendPort_, int receivePort_ = 50000, short timeToLive_ = 4, int timeOut_ = 5000) {

            string[] result= {};
            System.Collections.ArrayList stringArray = new System.Collections.ArrayList(result);

            IPAddress ipAddr = IPAddress.Parse(ipAddr_);    // マルチキャストアドレス
            int sendPort = sendPort_;                       // 送信先ポート
            int receivePort = receivePort_; 　　　　　　　　 // 送信元のポート(相手がこの番号に返してくる)

            //指定されたポートを起点に、使用されてないポート番号を探す
            bool alreadyinuse;
            do
            {
                alreadyinuse =
                (from p in
                     System.Net.NetworkInformation.IPGlobalProperties.GetIPGlobalProperties().GetActiveUdpListeners()
                 where p.Port == receivePort
                 select p).Count() == 1;

                if (alreadyinuse == true)
                {
                    receivePort = receivePort + 1;
                }

                if (receivePort == 65536) {
                    Debug.WriteLine("ポートが空いてない");
                    return null;
                }

            } while (alreadyinuse == true);

            Debug.WriteLine("使用ポート:" + receivePort);


            short timeToLive = timeToLive_;　　　　　　　　　   // TTL

            IPEndPoint distEP = new IPEndPoint(ipAddr, sendPort);             // 送信先(マルチキャストアドレス)を生成
            IPEndPoint remoteEP = new IPEndPoint(IPAddress.Any, receivePort); // 受信相手を生成

            sendJsonData snd = new sendJsonData(); // Json形式に変換するため
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(sendJsonData)); //
            MemoryStream ms = new MemoryStream();  //  

            // 検索用データを作成
            snd.Command = "Search_Everio2012Regular";

            serializer.WriteObject(ms, snd);            // JSON形式に変換
            byte[] sendBytes = ms.ToArray();             // memoryStreamからバイト配列に変換

            // UDPの検索パケット送信
            IPAddress ip_nic;
            IPEndPoint sourceEP;

            NetworkInterface[] netIFs = NetworkInterface.GetAllNetworkInterfaces();
            UnicastIPAddressInformationCollection ips;
            int num_of_IF = netIFs.Count();

            // 
            for (int loopI = 0; loopI < num_of_IF ; loopI++) {
                if (netIFs[loopI].OperationalStatus == OperationalStatus.Up) {
                    ips = netIFs[loopI].GetIPProperties().UnicastAddresses;

                    for (int k = 0; k < ips.Count; k++)
                    {
                        ip_nic = ips[k].Address;
                        if (ip_nic.AddressFamily == AddressFamily.InterNetwork){

                            if (ip_nic.ToString() != "127.0.0.1")
                            {
                                sourceEP = new IPEndPoint(ip_nic, receivePort);
                                sendUdp = new UdpClient(sourceEP);
                                sendUdp.Ttl = timeToLive;
                                sendUdp.Send(sendBytes, sendBytes.Length, distEP);
                            }
                        }
                    }
                }
            }


            // 規定回数ループを繰り返す
            int i = 0; // 検索ループ回数
            int j = 0; // 見つかったカメラの数

            byte[] receivedByte = { };
            DataContractJsonSerializer dcJS = new DataContractJsonSerializer(typeof(receiveJsonData)); //受信データを変換するため

            Thread thReceive = new Thread(delegate(){

                while (true) {

                    try
                    {
                        receivedByte = sendUdp.Receive(ref remoteEP);
                    }
                    catch (SocketException sockEx) {
                        Debug.WriteLine(sockEx);
                        if (sendUdp != null)
                        {
                            sendUdp.Close();
                        }
                        return;   
                    }

                    ms.Write(receivedByte, 0, receivedByte.Length);   // 受け取ったバイト配列をストリームに変換
                    ms.Seek(0, SeekOrigin.Begin);                     // 読み出し位置を先頭に戻す

                    var rcv = dcJS.ReadObject(ms) as receiveJsonData; // ストリーム(JSON形式)のデータからオブジェクトに変換
                    var dd = rcv.Data as Data;                       // 受信したJSONデータからDataフィールド以下を取得  

                    ms.SetLength(0);                                  // ストリームをクリア

                    if (receivedByte.Length > 0)
                    {
                        stringArray.Add(dd.Host + ":" + dd.Port );
                        j = j + 1;       
                        // Debug.Write(dd.CameraName + "  ");
                        Debug.WriteLine(dd.Host + ":" + dd.Port);
                    }
                    Thread.Sleep(10);
                    i = i + 1;
                
                };
            });
            thReceive.Start();

            Thread.Sleep(timeOut_);

            // 後始末
            ms.Close();

            if (sendUdp != null)
            {
                sendUdp.Close();
            }

            thReceive.Abort();
            distEP = null;
            remoteEP = null;
            ms = null;
            thReceive = null;
            sendUdp = null;
            dcJS = null;
            snd = null;
            serializer = null;

            Thread.Sleep(timeOut_);

            if (j == 0) {   //カメラが見つからない場合はnullを返す
                return null;
            }
            else
            {

                result = (string[]) stringArray.ToArray(typeof(string));
                return result;
            }
        }

    }
}
