﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Threading;
using System.Drawing;

namespace WindowsFormsApplication1
{
    class httpClass
    {

        // アクセス用URL各種
        public const string URL_GET_JPG = "/cgi-bin/get_jpg.cgi";
        public const string URL_MJPEG = "/cgi-bin/mjpeg.cgi";                       // MotionJPEG取得URL
        public const string URL_M2TS = "/cgi-bin/movie_sp.cgi";                     // Mpeg2ts取得URL
        public const string URL_LPCM = "/cgi-bin/lpcm.cgi";                         // LPCM取得URL
        public const string URL_GET_PTZ_POSITION = "/cgi-bin/ptz_position.cgi";     // PTZの位置を取得するためのURL
        public const string URL_SESSION_START = "/php/session_start.php" ;          // セッションを取得するためのURL(rootのセッション)
        public const string URL_SESSION_START_USER = "/php/session_start_user.php"; // セッションを取得するためのURL(camuserのセッション)
        public const string URL_CONTINUE_SESSION = "/php/session_continue.php";     // セッションを継続するためのURL 
        public const string URL_FINISH_SESSION = "/php/session_finish.php";         // セッションを終了するためのURL
        public const string URL_SEND_COMMAND = "/cgi-bin/cmd.cgi";                  // コマンド送信のためのURL
        public const string URL_GET_CAM_INFO = "/cgi-bin/hello.cgi";                // カメラの機能を取得するためのURL
        public const string URL_GET_CAM_STATUS = "/cgi-bin/camera_status.cgi";      // カメラの状態を取得するためのURL
        public const string URL_GET_CAM_AUTO_STATUS = "/cgi-bin/auto_status.cgi";   // カメラの設定(明るさ補正など)を取得するためのURL

        // タイムアウト等の数値
        public const int CONTINUE_SESSION_INTERVAL = 1000;                      // セッションを継続要求する間隔(ms)
        public const int CONTINUE_SESSION_COUNT = 15;                           // セッションを継続要求待ちを繰り返す回数(規定回数待つとセッション継続を要求する)
        public const int STREAM_TIMEOUT = 3000;                                 // HttpWebRequest発行後のタイムアウト



        // 指定URLからストリームを取得する
        public Stream getResponseStream(string url, int timeout = 5000, CookieContainer cookie = null, bool useProxy = false)
        {
            HttpWebRequest hWebReq;
            HttpWebResponse hWebRes;
            Stream objStream;

            hWebReq = (HttpWebRequest)HttpWebRequest.Create(url);
            if (cookie != null)
            {
                hWebReq.CookieContainer = cookie;
            }

            // プロキシを使用するかどうかのチェック
            if (useProxy == true)
            {
                hWebReq.Proxy = System.Net.WebRequest.GetSystemWebProxy();
            }
            else
            {
                hWebReq.Proxy = null;
            }

            hWebReq.Method = "GET";
            hWebReq.UserAgent = "UDP Sound Receiver";
            hWebReq.ReadWriteTimeout = timeout;
            hWebReq.Timeout = timeout; //タイムアウト時間(ms)
            hWebReq.KeepAlive = true;

            // レスポンスを取得
            try
            {
                hWebRes = (HttpWebResponse)hWebReq.GetResponse();
                objStream = hWebRes.GetResponseStream();
            }
            catch (System.Net.WebException e)
            {
                dispError("getResponseStream", url, e);
                objStream = null;　// エラー発生時にはnullを返す
            }
            finally {
            }

            return objStream;

        }


        // 指定URLにデータを送信する //
        public Stream postStream(string url, byte[] sendBytes, CookieContainer cookie_ = null, bool useProxy = false)
        {

            HttpWebRequest hWebReq;
            Stream sndStream;   //送信用ストリーム
            Stream objStream;   //受信用ストリーム

            // データ送信
            hWebReq = (HttpWebRequest)HttpWebRequest.Create(url);

            if (cookie_ != null)
            {
                hWebReq.CookieContainer = cookie_;
            }

            // プロキシを使用するかどうかのチェック
            if (useProxy == true)
            {
                hWebReq.Proxy = System.Net.WebRequest.GetSystemWebProxy();
            }
            else
            {
                hWebReq.Proxy = null;
            }

            hWebReq.ContentType = "application/json";
            hWebReq.Method = "POST";
            hWebReq.Timeout = STREAM_TIMEOUT;
            hWebReq.ContentLength = sendBytes.Length;
            hWebReq.ServicePoint.Expect100Continue = false;

            // データ送信
            try
            {
                sndStream = hWebReq.GetRequestStream();
                sndStream.Write(sendBytes, 0, sendBytes.Length);
                sndStream.Close();
            }
            catch (WebException wex)
            {
                System.Diagnostics.Debug.WriteLine(wex);
            };

            // データ受信
            try
            {
                objStream = hWebReq.GetResponse().GetResponseStream();
                return objStream;
            }
            catch (System.Net.WebException e)
            {
                dispError("postStream", url, e);
                return null;
            }
            finally
            {
                hWebReq.Abort();                        // 普通に終わるとHTTPセッションがしばらく切れないため
            }

        }


        // 初期セッションの取得(外部からは通常はこちらを使う)
        public CookieContainer getSession() 
        {
            int cam = MainForm.Form1Instance.selectedCamera;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];
            cameraSetting camS = MainForm.Form1Instance.cameraSettings[cam];
            string userName = camS.userName;
            string passWord = camS.passWord;
            bool useProxy = camS.useProxy;
            string url = "http://" + userName + ":" + passWord+ "@" + camS.ipAddress + ":" + camS.portNumber + URL_SESSION_START;

            return getSession(url, cookie, userName, passWord, useProxy);
        }


        // セッションの更新処理 (外部からは通常こちらを使う) //
        public void updateSession()
        {
            int cam = MainForm.Form1Instance.selectedCamera;
            cameraSetting camS = MainForm.Form1Instance.cameraSettings[cam];
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];
            string url = camS.ipAddress + ":" + camS.portNumber;
            string userName = camS.userName;
            string passWord = camS.passWord;
            bool useProxy = camS.useProxy;
            updateSession(url, cookie, userName, passWord, useProxy, cam);
        }


        // セッションの終了処理 (現在選択中のカメラを操作する場合) (外部からは通常はこちらを使う)
        public void endSession() {
            int cam = MainForm.Form1Instance.selectedCamera;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];
            cameraSetting camS = MainForm.Form1Instance.cameraSettings[cam];
            string userName = camS.userName;
            string passWord = camS.passWord;
            bool useProxy = camS.useProxy;
            string url = "http://" + userName + ":" + passWord + "@" + camS.ipAddress + ":" + camS.portNumber + URL_FINISH_SESSION;
            getSession(url, cookie, userName, passWord, useProxy);
        }


        // セッションの終了処理(カメラの番号を指定する場合)(外部からは通常はこちらを使う)
        public void endSession(int cam) {
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];
            cameraSetting camS = MainForm.Form1Instance.cameraSettings[cam];
            string userName = camS.userName;
            string passWord = camS.passWord;
            bool useProxy = camS.useProxy;
            string url = "http://" + userName + ":" + passWord + "@" + camS.ipAddress + ":" + camS.portNumber + URL_FINISH_SESSION;
            getSession(url, cookie, userName, passWord, useProxy);
        
        }


        // 現在のPTZの値を読み取る(通常は外部からはこちらを使う)
        public PresetData_without_Name GetPTZ()
        {
            int cam = MainForm.Form1Instance.selectedCamera;
            string url = "http://" + MainForm.Form1Instance.cameraSettings[cam].ipAddress + ":" + MainForm.Form1Instance.cameraSettings[cam].portNumber + httpClass.URL_GET_PTZ_POSITION;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];
            bool useProxy = MainForm.Form1Instance.cameraSettings[cam].useProxy;
            return GetPTZ(url, cookie, useProxy);
        }


        // カメラ状態を取得して、メインフォームの表示を更新する(外部からは通常はこちらを使う)
        public void getCameraStatus() {
            int cam = MainForm.Form1Instance.selectedCamera;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];
            cameraSetting camS = MainForm.Form1Instance.cameraSettings[cam];
            bool useProxy = camS.useProxy;
            string url = camS.ipAddress + ":" +camS.portNumber;
            getCameraStatus(url, cookie, useProxy, cam);
        }


        // カメラの状態(明るさ補正など)を取得する (通常は外部からはこちらを使う
        public Cam_AutoStatus getCamAutoStatus()
        {
            int cam = MainForm.Form1Instance.selectedCamera;
            string url = "http://" + MainForm.Form1Instance.cameraSettings[cam].ipAddress + ":" + MainForm.Form1Instance.cameraSettings[cam].portNumber + httpClass.URL_GET_CAM_AUTO_STATUS;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam]; bool useProxy = MainForm.Form1Instance.cameraSettings[cam].useProxy;
            return getCamAutoStatus(url, cookie, useProxy);
        }


        // カメラの機能を取得する (通常は外部からはこちらを使う)
        public Cam_Feature_Data getCamFeature_Data()
        {
            int cam = MainForm.Form1Instance.selectedCamera;
            string url = "http://" + MainForm.Form1Instance.cameraSettings[cam].ipAddress + ":" + MainForm.Form1Instance.cameraSettings[cam].portNumber + httpClass.URL_GET_CAM_INFO;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];
            bool useProxy = MainForm.Form1Instance.cameraSettings[cam].useProxy;

            return getCamFeature_Data(url, cookie, useProxy);
        }


        // エラー内容をコンソールに表示する関数(デバッグ用) //
        public void dispError(string methodName, string url, Exception ex)
        {
            Console.WriteLine(methodName + "でエラー発生");
            Console.WriteLine("URL = " + url);
            Console.WriteLine("エラー = " + ex);
        }


        ////////////////////////////////////////////
        ////////// 通常は外部からは使わない //////////
        ////////////////////////////////////////////

        // 初期セッションの取得
        private CookieContainer getSession(string url, CookieContainer cook_org = null, string user_ = "root", string pass_ = "password", bool useProxy = false)
        {

            CookieContainer cookie_; // 取得したクッキーを格納する場所
            HttpWebRequest hWebReq;  // サーバーに発行するHTTPリクエスト
            HttpWebResponse hWebRes; // サーバーから取得するHTTPレスポンス
            Stream ns;

            // クッキーが指定されてない場合は クッキーを新規で作成、指定されている場合はそれを使う
            if (cook_org == null)
            {
                cookie_ = new CookieContainer();
            }
            else
            {
                cookie_ = cook_org;
            }

            hWebReq = (HttpWebRequest)HttpWebRequest.Create(url);

            // プロキシを使用するかどうかのチェック
            if (useProxy == true)
            {
                hWebReq.Proxy = System.Net.WebRequest.GetSystemWebProxy();
            } else {
                hWebReq.Proxy = null;
            }

            hWebReq.Credentials = new NetworkCredential(user_, pass_);
            hWebReq.Timeout = STREAM_TIMEOUT;
            hWebReq.Method = "GET";
            hWebReq.CookieContainer = cookie_;

            try
            {
                hWebRes = (HttpWebResponse)hWebReq.GetResponse();
                ns = hWebRes.GetResponseStream();　                      // ストリームの中身は実際には不要なのでコマンドだけ投げる
                System.Diagnostics.Debug.WriteLine("cookie更新完了");    // 
            }
            catch {
                throw;
            }
            finally {
                hWebReq.Abort();            // 普通に終わるとHTTPセッションがしばらく切れず、連続でアクセスするとタイムアウトを返すため、対策として
            }

            return cookie_;

        }


        // セッションの更新処理 //
        private void updateSession(string url_, CookieContainer cookie_, string userName_, string passWord_, bool useProxy_, int cam_)
        {
            int cam = cam_;
            int count = 0;
            int count_max = httpClass.CONTINUE_SESSION_COUNT;

            while (MainForm.Form1Instance.receiveFlags[cam] == true)
            {
                if (count < count_max)
                {
                    count = count + 1;
                }
                else
                {
                    try
                    {
                        getSession(@"http://" + url_ + httpClass.URL_CONTINUE_SESSION, cookie_, userName_, passWord_, useProxy_);
                    }
                    catch (WebException wex)
                    {
                        dispError("Continue_Session", @"http://" + url_ + httpClass.URL_CONTINUE_SESSION, wex);
                    }
                    count = 0;
                }
                Thread.Sleep(httpClass.CONTINUE_SESSION_INTERVAL);
            }
        }


        public PresetData_without_Name GetPTZ(int cam)
        {
            string url = "http://" + MainForm.Form1Instance.cameraSettings[cam].ipAddress + ":" + MainForm.Form1Instance.cameraSettings[cam].portNumber + httpClass.URL_GET_PTZ_POSITION;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];
            bool useProxy = MainForm.Form1Instance.cameraSettings[cam].useProxy;
            return GetPTZ(url, cookie, useProxy);
        }


        // 現在のPTZの値を読み取る //
        private PresetData_without_Name GetPTZ(string url, CookieContainer cookie, bool useProxy)
        {
            PresetData_without_Name preData = new PresetData_without_Name();
            Stream objStream;
            JSON_Commands jsonC = new JSON_Commands();
            byte[] buff = new Byte[500];    // ストリーム取得用のバッファ

            try
            {
                objStream = getResponseStream(url, httpClass.STREAM_TIMEOUT, cookie, useProxy);
            }
            catch (WebException wex)
            {
                if (wex.Status == WebExceptionStatus.Timeout)
                {
                    System.Diagnostics.Debug.WriteLine("タイムアウト");
                }
                objStream = null;
            }

            if (objStream != null)
            {
                int readByte = objStream.Read(buff, 0, buff.Count());
                preData = jsonC.getResponsePTZ(buff, readByte);
            }

            return preData;
        }



        // カメラの状態(明るさ補正など)を取得する
        private Cam_AutoStatus getCamAutoStatus(string url_, CookieContainer cookie_, bool useProxy_)
        {            
            Cam_AutoStatus cAS = new Cam_AutoStatus();
            Stream objStream;
            MemoryStream ms = new MemoryStream();
            JSON_Commands jsonC = new JSON_Commands();

            byte[] buff = new Byte[1024];    // ストリーム取得用のバッファ

            try
            {
                objStream = getResponseStream(url_, httpClass.STREAM_TIMEOUT, cookie_, useProxy_);
            }
            catch (WebException wex)
            {
                if (wex.Status == WebExceptionStatus.Timeout)
                {
                    System.Diagnostics.Debug.WriteLine("タイムアウト");
                }
                objStream = null;
            }

            if (objStream != null)
            {
                int readByte = objStream.Read(buff, 0, buff.Count());
                cAS = jsonC.getResponseCamAutoStatus(buff, readByte);
            }

            return cAS;

        }


        // カメラ状態を取得して、メインフォームの表示を更新する関数 //
        private void getCameraStatus(string url_, CookieContainer cookie_, bool useProxy_, int cam_) 
        {
            int interval = 200;
            int count = 5;
            int count_max = 5;
            PresetData_without_Name pdwn = new PresetData_without_Name();

            while (MainForm.Form1Instance.receiveFlags[cam_] == true )
            {
                //
                if (count == count_max)
                {
                    MainForm.Form1Instance.camStatuses[cam_] = getCamStatus(@"http://" + url_ + httpClass.URL_GET_CAM_STATUS, cookie_, useProxy_);
                    pdwn = GetPTZ(cam_);
                    count = 0;
                }
                else {

                    count = count + 1;
                }

               if (cam_ == MainForm.Form1Instance.selectedCamera) {

                   string drawString = "";

                   // フォーカス状態の取得
                   if (MainForm.Form1Instance.camStatuses[cam_].IsFocusAuto)
                   {
                       drawString = "Focus : Auto   ";
                   } else {
                       drawString = "Focus : Manual ";
                   }
                   
                   drawString = drawString + " Zoom : " + string.Format("{0, -4}", (((double)pdwn.DeciZoomPosition) / 10)) + " ";  // 4桁確保で左詰め表示

                   if (pdwn.PanPosition > 0)
                   {
                       drawString = drawString + " Pan : → " + string.Format("{0, -3}", pdwn.PanPosition) + " ";   // 3桁確保で左詰表示(右詰のときはマイナスを取る)
                   }

                   if (pdwn.PanPosition < 0)
                   {
                       drawString = drawString + " Pan : ← " + string.Format("{0, -3}",(-1) * pdwn.PanPosition) + " ";
                   }                  
                   if (pdwn.PanPosition == 0) 
                   {
                       drawString = drawString + " Pan : 0     ";
                   }
                   if (pdwn.TiltPosition > 0)
                   {
                       drawString = drawString + " Tilt : ↑ " + string.Format("{0, -2}", pdwn.TiltPosition) + " ";
                   }
                   if (pdwn.TiltPosition < 0) {
                       drawString = drawString + " Tilt : ↓ " + string.Format("{0, -2}", (-1) * pdwn.TiltPosition) + " ";
                   }
                   if (pdwn.TiltPosition == 0)
                   {
                       drawString = drawString + " Tilt : 0     ";
                   }
                   if (MainForm.Form1Instance.camStatuses[cam_].CamMotion == "Normal")
                   {
                       drawString = drawString + " ■ STOP ";
                   }
                   else
                   {
                       drawString = drawString + " ● REC  ";
                   }

                   switch (MainForm.Form1Instance.camStatuses[cam_].SdcardExist){
                       case "NoCard":
                            drawString = drawString + "   SD : ----      ";
                            break;
                       case "WriteProtect":
                            drawString = drawString + "   SD : LOCKED ";
                            break;
                       case "Exist":
                            drawString = drawString + "   SD :" +MainForm.Form1Instance.camStatuses[cam_].SdcardRemains + " ";
                            break;
                   }

                   int sW = 640;
                   int sH = 20;

                   Bitmap bmpdata = new Bitmap(sW, sH);
                   Graphics gra = Graphics.FromImage(bmpdata);
                   gra.FillRectangle(Brushes.Black, 0, 0, sW, sH);
                   gra.DrawString(drawString, System.Drawing.SystemFonts.DefaultFont, Brushes.White, 3, 3);
                   gra.Dispose();
                   MainForm.Form1Instance.picturebox_cameraStatus = bmpdata;

                }
                Thread.Sleep(interval);
            }
        }


        // カメラの状態を取得して、構造体Cam_Statusに格納する関数 //
        private Cam_Status getCamStatus(string url, CookieContainer cookie, bool useProxy)
        {
            Cam_Status cSD = new Cam_Status();
            Stream objStream;
            JSON_Commands jsonC = new JSON_Commands();

            byte[] buff = new Byte[1024];    // ストリーム取得用のバッファ

            try
            {
                objStream = getResponseStream(url, httpClass.STREAM_TIMEOUT, cookie, useProxy);
            }
            catch (WebException wex)
            {
                if (wex.Status == WebExceptionStatus.Timeout)
                {
                    System.Diagnostics.Debug.WriteLine("タイムアウト");
                }
                objStream = null;
            }

            if (objStream != null)
            {
                int readByte = objStream.Read(buff, 0, buff.Count());
                cSD = jsonC.getResponseCamStatus(buff, readByte);
            }

            return cSD;
        }


        // カメラの機能を取得する //
        private Cam_Feature_Data getCamFeature_Data(string url, CookieContainer cookie, bool useProxy)
        {
            Cam_Feature_Data cFD = new Cam_Feature_Data();
            Stream objStream;
            JSON_Commands jsonC = new JSON_Commands();

            byte[] buff = new Byte[1024];    // ストリーム取得用のバッファ

            try
            {
                objStream = getResponseStream(url, httpClass.STREAM_TIMEOUT, cookie, useProxy);
            }
            catch (WebException wex)
            {
                if (wex.Status == WebExceptionStatus.Timeout)
                {
                    System.Diagnostics.Debug.WriteLine("タイムアウト");
                }
                objStream = null;
            }

            if (objStream != null)
            {
                int readByte = objStream.Read(buff, 0, buff.Count());
                cFD = jsonC.getResponseCamFeature(buff, readByte);
            }

            return cFD;
        }



    }

}
