﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.DirectX.DirectInput;
using System.Threading;
using System.Net;
using System.Diagnostics;

namespace WindowsFormsApplication1
{

    class joyPadClass
    {
        public const int PAD_DIRECTION_CENTER = 32768;      // 方向キーセンターの値
        public const int PAD_DIRECTION_DEADZONE = 16384;    // 方向キーの無効領域(アナログパッド用)
        public const int PAD_DIRECTION_MAX = 65535;         // 方向キーの右端の値
        public const int PAD_DIRECTION_ACT_PLUS = PAD_DIRECTION_CENTER + PAD_DIRECTION_DEADZONE;
        public const int PAD_DIRECTION_ACT_MINUS = PAD_DIRECTION_CENTER - PAD_DIRECTION_DEADZONE;
        public const int PAD_DIRECTION_MIN = 0;             // 方向キーの左端の値
        public const int PAD_BUTTON_RELEASE = 0;    // ボタンが押されていないときの戻り値
        public const int PAD_BUTTON_PUSHED = 128;   // ボタンが押されているときの戻り値
        public const int PAD_SCAN_INTERVAL = 75;    // パッドのスキャン間隔

        public const int PAD_DEFAULT = 0;
        public const int PAD_DEFAULT_PANTILT_SPEED_UP = 0;
        public const int PAD_DEFAULT_PANTILT_SPEED_DOWN = 1;
        public const int PAD_DEFAULT_Z_ZOOM = 2;
        public const int PAD_DEFAULT_W_ZOOM = 3;
        public const int PAD_DEFAULT_CENTER_RESET = 4;

        private int[] PAN_SPEEDS_RIGHT = { constClass.CMD_PAN_STOP, constClass.CMD_PAN_RIGHT_MIN, constClass.CMD_PAN_RIGHT_MID, 
                                           constClass.CMD_PAN_RIGHT_HIG, constClass.CMD_PAN_RIGHT_HI2, constClass.CMD_PAN_RIGHT_MAX };
        private int[] PAN_SPEEDS_LEFT = { constClass.CMD_PAN_STOP, constClass.CMD_PAN_LEFT_MIN, constClass.CMD_PAN_LEFT_MID, 
                                          constClass.CMD_PAN_LEFT_HIG, constClass.CMD_PAN_LEFT_HI2, constClass.CMD_PAN_LEFT_MAX };
        private int[] TILT_SPEEDS_UP = { constClass.CMD_TILT_STOP, constClass.CMD_TILT_UP_MIN, constClass.CMD_TILT_UP_MID, 
                                         constClass.CMD_TILT_UP_HIG, constClass.CMD_TILT_UP_HI2, constClass.CMD_TILT_UP_MAX };
        private int[] TILT_SPEEDS_DOWN = { constClass.CMD_TILT_STOP, constClass.CMD_TILT_DOWN_MIN, constClass.CMD_TILT_DOWN_MID, 
                                           constClass.CMD_TILT_DOWN_HIG, constClass.CMD_TILT_DOWN_HI2, constClass.CMD_TILT_DOWN_MAX };


        // 最初に見つかったジョイスティックを返す        
        public Microsoft.DirectX.DirectInput.Device getFirstJoyPad(){

            DeviceList devList = Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly);    // パッドの一覧を取得
            Microsoft.DirectX.DirectInput.Device joystick = null;                                                      // パッド

            // 全てのコントローラを列挙
            foreach (DeviceInstance dev in devList)
            {
                joystick = new Microsoft.DirectX.DirectInput.Device(dev.InstanceGuid);
                joystick.SetCooperativeLevel(MainForm.Form1Instance.handler, CooperativeLevelFlags.Background | CooperativeLevelFlags.NonExclusive);
                break;  // 最初に見つかったジョイスティックを操作対象とする
            };
            return joystick;
        }


        // 指定した番号のジョイスティックを返す(存在しない場合はnullを返す)
        public Microsoft.DirectX.DirectInput.Device getJoyPad(int joyPadNumber, IntPtr wHandler) {

            DeviceList devList = Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly);    // パッドの一覧を取得
            Microsoft.DirectX.DirectInput.Device joystick = null;
            int i = 0;

            if (devList.Count < joyPadNumber)
            {
                return joystick;
            }

            foreach (DeviceInstance dev in devList){
                if (i == joyPadNumber) {
                    joystick = new Microsoft.DirectX.DirectInput.Device(dev.InstanceGuid);
                    joystick.SetCooperativeLevel(wHandler, CooperativeLevelFlags.Background | CooperativeLevelFlags.NonExclusive);
                    break;
                }
                i = i + 1;
            }

            return joystick;
        }



        // ゲームコントローラの数を取得する
        public int getNumber()
        {
            return Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly).Count; // ゲームコントローラのデバイス一覧を取得            
        }


        // コントローラーの入力を取得する関数
        public void getControl()
        {
            pads pk = new pads();
            pk = MainForm.Form1Instance.padKeys;

            httpClass htC = new httpClass();
            ptzClass ptzC = new ptzClass();
            int selectedCamera;
            bool inversePicture;
            
            Microsoft.DirectX.DirectInput.Device joyStick = MainForm.Form1Instance.joyStick;
            int num_of_buttons = joyStick.Caps.NumberButtons; //ボタンの数を取得

            if (joyStick == null)
            { // joyStickが無い場合は何もしない
                return;
            };

            int x_speed = 0;
            int previous_x_speed = 0;
            int y_speed = 0;
            int previous_y_speed = 0;
            int multiple = 0;
            int zoom_speed = 0;
            int previous_zoom_speed = 0;

            while (MainForm.Form1Instance.isAnyConnection == true)
            {

                selectedCamera = MainForm.Form1Instance.selectedCamera;
                inversePicture = MainForm.Form1Instance.cameraSettings[selectedCamera].inversePicture;

                try
                {

                    joyStick.Poll();
                    // コントローラの状態をポーリングで取得
                    JoystickState state = joyStick.CurrentJoystickState;

                    // ボタンの数が、各ボタンに設定されている番号より大きい場合だけ実行する
                    // 加速/減速ボタンの動作
                    if (num_of_buttons > pk.PanTilt_Up & num_of_buttons > pk.PanTilt_Down) {
                        multiple = (state.GetButtons()[pk.PanTilt_Up] - state.GetButtons()[pk.PanTilt_Down]) / joyPadClass.PAD_BUTTON_PUSHED;
                    }

                    // ズームT/Wボタンの動作
                    if (num_of_buttons > pk.T_Zoom & num_of_buttons > pk.W_Zoom)
                    {
                        zoom_speed = (state.GetButtons()[pk.T_Zoom] - state.GetButtons()[pk.W_Zoom]) / joyPadClass.PAD_BUTTON_PUSHED * MainForm.Form1Instance.zoom_Speed;
                    } else {
                        zoom_speed = 0;
                    }

                    if (zoom_speed != previous_zoom_speed)
                    {
                        previous_zoom_speed = zoom_speed;
                        ptzC.postZoomCommand(zoom_speed);
                        Thread.Sleep(joyPadClass.PAD_SCAN_INTERVAL);                // キーを連続で送るとタイムアウトになる可能性が高いため、送り終わりを待つ
                    }

                    // センターリセットボタンの動作
                    if (num_of_buttons > pk.Center_Reset) 
                    {
                        if (state.GetButtons()[pk.Center_Reset] == joyPadClass.PAD_BUTTON_PUSHED)
                        {
                            ptzC.postPTCommand(constClass.CMD_PAN_TILT_ABSOLUTE, 0, 0, false);
                            Thread.Sleep(joyPadClass.PAD_SCAN_INTERVAL);                // キーを連続で送るとタイムアウトになる可能性が高いため、送り終わりを待つ
                        }
                    }

                    // 方向キーの動作
                    int rangeX = (joyPadClass.PAD_DIRECTION_MAX - joyPadClass.PAD_DIRECTION_ACT_PLUS) / constClass.PAN_SPEED_STEP;  // X方向の入力値を取得
                    int rangeY = (joyPadClass.PAD_DIRECTION_MAX - joyPadClass.PAD_DIRECTION_ACT_PLUS) / constClass.TILT_SPEED_STEP; // Y方向の入力値を取得

                    if (state.X > joyPadClass.PAD_DIRECTION_ACT_PLUS)   // 横方向の計算(+方向に倒した場合)
                    {
                        x_speed = ((state.X - joyPadClass.PAD_DIRECTION_ACT_PLUS) / rangeX);
                        x_speed = PAN_SPEEDS_RIGHT[x_speed + multiple + 1];
                    }
                    if (state.X < joyPadClass.PAD_DIRECTION_ACT_MINUS)  // 横方向の計算(-方向に倒した場合)
                    {
                        x_speed = ((joyPadClass.PAD_DIRECTION_ACT_MINUS - state.X) / rangeX);
                        x_speed = PAN_SPEEDS_LEFT[x_speed + multiple + 1];
                    }
                    if (joyPadClass.PAD_DIRECTION_ACT_PLUS >= state.X & state.X >= joyPadClass.PAD_DIRECTION_ACT_MINUS) // 横方向に倒れてない場合(アナログを考慮して、一定範囲は反応しないようにする)
                    {
                        x_speed = PAN_SPEEDS_LEFT[0];
                    }

                    if (state.Y > joyPadClass.PAD_DIRECTION_ACT_PLUS)   // 縦方向の計算(+方向に倒した場合)
                    {
                        y_speed = ((state.Y - joyPadClass.PAD_DIRECTION_ACT_PLUS) / rangeY);
                        y_speed = TILT_SPEEDS_DOWN[y_speed + multiple + 1];
                    }
                    if (state.Y < joyPadClass.PAD_DIRECTION_ACT_MINUS)  // 横方向の計算(-方向に倒した場合)
                    {
                        y_speed = ((joyPadClass.PAD_DIRECTION_ACT_MINUS - state.Y) / rangeY);
                        y_speed = TILT_SPEEDS_UP[y_speed + multiple +1];
                    }
                    if (joyPadClass.PAD_DIRECTION_ACT_PLUS >= state.Y & state.Y >= joyPadClass.PAD_DIRECTION_ACT_MINUS) // 横方向に倒れていない場合(アナログを考慮して、一定範囲は反応しないようにする)
                    {
                        y_speed = TILT_SPEEDS_DOWN[0];
                    }
                    
                    if (x_speed != previous_x_speed)    // パン(横方向)のコマンド送信
                    {                                   // 通信量を減らすため、変化が無い場合は送信しない
                        previous_x_speed = x_speed;
                        ptzC.postPTCommand(x_speed, 0, 0, inversePicture);
                    }

                    Thread.Sleep(joyPadClass.PAD_SCAN_INTERVAL);             // キーを連続で送るとタイムアウトになる可能性が高いため、送り終わりを待つ

                    if (y_speed != previous_y_speed)    // チルト(縦方向)のコマンド送信
                    {                                   // 通信量を減らすため、変化が無い場合は送信しない
                        previous_y_speed = y_speed;
                        ptzC.postPTCommand(y_speed, 0, 0, inversePicture);
                    }


                }
                catch (Exception ex)
                {
                    Debug.WriteLine(ex);
                }

                Thread.Sleep(joyPadClass.PAD_SCAN_INTERVAL);

            }
            MainForm.Form1Instance.isControllerRunning = false;

        }


        //Debug.WriteLine("名称: " + joystick.DeviceInformation.ProductName + Environment.NewLine);   
        //Debug.WriteLine("軸の数: " + joystick.Caps.NumberAxes + Environment.NewLine); // コントローラの軸の数
        //Debug.WriteLine("PoVハットの数: " + joystick.Caps.NumberPointOfViews + Environment.NewLine); // PoV hatの数



    }
}
