﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Threading;

namespace WindowsFormsApplication1
{   
    class jpgClass
    {

        // ストリームから受け取ったデータから、MotionJPEGのヘッダ情報を取り出すための構造体
        public struct headerDatas
        {
            public bool Valid;
            public int headerStart;
            public int headerSize;
            public int jpgStart;
            public int jpgSize;
            public int jpgEnd;
        }

        public const int HEADER_LENGTH_MAX = 80; // ヘッダーの最大長さ(実際は70以下だが、とりあえず80)        
        public const int BLUE = 0x01;
        public const int BLACK = 0x02;


        // 外部から呼ぶ場合の関数 ///////////////////////////////////////////
        ////// MotionJpegの映像を受け取る (外からは通常こちらを使用する) ///// 
        public void getMjpeg() {

            int cam_ = MainForm.Form1Instance.selectedCamera;
            string ipAddr = MainForm.Form1Instance.cameraSettings[cam_].ipAddress;
            string port = MainForm.Form1Instance.cameraSettings[cam_].portNumber;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam_];
            
            getMjpeg(ipAddr,port,cookie,cam_);
        }


        ///// サムネイル画像を更新する関数
        public void updateThumbnail(int interval_ = 10000)
        {

            while (MainForm.Form1Instance.isAnyConnection == true)
            {
                for (int i = 0; i < constClass.MAX_NUM_OF_CAMERA; i++)
                {
                    if (MainForm.Form1Instance.receiveFlags[i] == true)
                    {
                        getJPG(MainForm.Form1Instance.cameraSettings[i].ipAddress, int.Parse(MainForm.Form1Instance.cameraSettings[i].portNumber), MainForm.Form1Instance.cookieContainers[i], i);
                    }
                    //else {
                    //    muteVideoScreen(BLACK, true, i);
                    //}   
                }
                Thread.Sleep(interval_);
            }
        }


        ///// スクリーンを塗りつぶす関数
        public void muteVideoScreen(int color, bool thumb_ = false, int cam_ = 0)
        {
            // thumb_がTrueの場合は本画面ではなく、サムネイル画面を操作する。
            // cam_はカメラ番号 (0～3)、範囲外を設定時はすべてのサムネイルを操作する
            Brush br;

            switch (color)
            {
                case 0x01:
                    br = Brushes.Blue;
                    break;
                case 0x02:
                    br = Brushes.Black;
                    break;
                default:
                    br = Brushes.White;
                    break;
            }

            if (thumb_ == false)
            {   // 通常のスクリーンの場合の処理
                int sW = constClass.SCREEN_WIDTH;
                int sH = constClass.SCREEN_HEIGHT;

                Bitmap bmpdata = new Bitmap(sW, sH);
                Graphics gra = Graphics.FromImage(bmpdata);
                gra.FillRectangle(br, 0, 0, sW, sH);
                MainForm.Form1Instance.screen = bmpdata;
            }
            else
            {
                // サムネイル用の画像の場合の処理
                int tW = constClass.THUMB_WIDTH;
                int tH = constClass.THUMB_HEIGHT;

                Bitmap bmpdata = new Bitmap(tW, tH);
                Graphics gra = Graphics.FromImage(bmpdata);
                gra.FillRectangle(br, 0, 0, tW, tH);

                switch (cam_)
                {
                    case 0:
                        MainForm.Form1Instance.thumbCam1 = bmpdata;
                        break;
                    case 1:
                        MainForm.Form1Instance.thumbCam2 = bmpdata;
                        break;
                    case 2:
                        MainForm.Form1Instance.thumbCam3 = bmpdata;
                        break;
                    case 3:
                        MainForm.Form1Instance.thumbCam4 = bmpdata;
                        break;
                    default:
                        MainForm.Form1Instance.thumbCam1 = bmpdata;
                        MainForm.Form1Instance.thumbCam2 = bmpdata;
                        MainForm.Form1Instance.thumbCam3 = bmpdata;
                        MainForm.Form1Instance.thumbCam4 = bmpdata;
                        break;
                }
            }
            return;
        }


        ///// スクリーンの内容をJPGとして保存し、その際のファイル名を返す関数
        public string saveJPG()
        {
            string fPath;
            string postfix;
            string fileName;
            const int fileNum = 10; //保存可能なファイルの数

            // プログラムの実行パスを取得
            fPath = MainForm.Form1Instance.pathes.applicationPath;

            // ファイルに付与する年月日時刻を取得
            postfix = string.Format("{0:0000}", DateTime.Now.Year) + string.Format("{0:00}", DateTime.Now.Month) +
                string.Format("{0:00}", DateTime.Now.Hour) + string.Format("{0:00}", DateTime.Now.Month) +
                string.Format("{0:00}", DateTime.Now.Minute) + string.Format("{0:00}", DateTime.Now.Second);

            fileName = "Saved_" + postfix + ".jpg";

            // ファイルが規定数ある場合、古い物を消す
            IEnumerable<string> files = Directory.GetFiles(fPath, "Saved_*");

            string[] fileList = files.ToArray();

            if (files.Count() >= fileNum)
            {
                System.IO.File.Delete(fileList[0]);
            }

            System.Drawing.Image im = MainForm.Form1Instance.screen;                

            //imの元はPictureBox1で、非同期に保存しようとすると汎用エラーで落ちるため、別のイメージを作成して、それを保存する
            Image img = new Bitmap(im);
            img.Save(fPath + fileName, System.Drawing.Imaging.ImageFormat.Jpeg);
            return fPath + fileName;
        }



        // 外部からは使用しない関数 ////////////////////////////////////////////////////////////////
        ////// MotionJpegの映像を受け取る ///// 
        private void getMjpeg(string ipAddr, string portNum, CookieContainer cookie, int cam_)
        {
            int cam = cam_;

            // 受信コード
            byte[] headBytes = new byte[171];
            byte[] tmpBytes = new byte[81920];
            byte[] initByte = { };
            byte[] nsBytes = new byte[20480];   // NetworkStream を受け取る配列
            byte[] nstmpBytes = new byte[2048]; // NetworkStream を受け取る配列(getResponseStreamから受け取ったあとは、MemoryStreamに一旦蓄積、そのあとnsBytesに移す
            int resSize;
           
            int notReceived = 0;    // データを受信しなかった回数をカウントする変数

            // JPG保存用の変数
            int count = 8;      // データの受信待ち回数
            jpgClass.headerDatas headData = new jpgClass.headerDatas();     // ヘッダの開始位置やJPGのサイズ、位置を保存する構造体

            byte[] newJpgByte;
            MemoryStream ms = new MemoryStream();
            MemoryStream ms2 = new MemoryStream();
            MemoryStream msToImg = new MemoryStream();

            // 一旦配列を初期化する //
            tmpBytes = initByte;
            nsBytes = initByte;

            Stream objStream;
            HttpWebRequest hWebReq;

        doloop: // エラー発生時にWebRequestを生成しなおして、再度取得開始する

            hWebReq = (HttpWebRequest)HttpWebRequest.Create("http://" + ipAddr + ":" + portNum + httpClass.URL_MJPEG);
            hWebReq.CookieContainer = cookie;
            hWebReq.Method = "GET";
            hWebReq.UserAgent = "UDP Sound Receiver";
            hWebReq.Timeout = 5000;
            hWebReq.KeepAlive = true;


            int i;

            while (MainForm.Form1Instance.receiveFlags[cam] == true)
            {
                // カメラが接続されていないときは映像ストリームの受信を止める
                while (cam != MainForm.Form1Instance.selectedCamera & MainForm.Form1Instance.receiveFlags[cam] == true)   
                {
                    tmpBytes = initByte;
                    if (MainForm.Form1Instance.receiveFlags[cam] == false)
                    {
                        hWebReq.Abort();
                        return;
                    }
                    Thread.Sleep(50);
                }

                resSize = 0;
                nsBytes = initByte;
                ms2.SetLength(0);

                hWebReq.CookieContainer = cookie;

                // HttpWebRequestからストリームを取得する (nstmpBytes -> MemoryStream -> nsBytes) //
                for (i = 0; i < count; i++)     // カウント数分のデータをバッファリングする
                {
                    try
                    {
                        objStream = hWebReq.GetResponse().GetResponseStream();  // ストリームを取得
                        resSize = objStream.Read(nstmpBytes, 0, nstmpBytes.Length);   // バイト配列に読み取り
                    }
                    catch (WebException wex)
                    {
                        System.Diagnostics.Debug.WriteLine(wex);
                        resSize = 0;
                    }

                    if (resSize > 0)
                    {
                        ms2.Write(nstmpBytes, 0, resSize);
                    }
                }

                nsBytes = ms2.ToArray();    // バッファリングしたデータを配列に変換

                if (nsBytes.Length != 0)
                {

                    tmpBytes = tmpBytes.Concat(nsBytes).ToArray();  // 前に使わずに残っていたデータと、新しく受信したデータを結合する
                    headData = getDatas_from_header(tmpBytes); // バイトデータからヘッダー情報を読み取る

                    if (headData.Valid == true) // 有効なヘッダー情報が取れた場合だけ処理する
                    { 
                        if ((tmpBytes.Length > headData.jpgEnd) & (headData.jpgSize != 0))  // データのサイズがJPG１枚分無い場合は、何もせずストリーム受信へ
                        {
                            newJpgByte = new byte[headData.jpgSize];                  // JPGデータの格納先
                            Array.Copy(tmpBytes, headData.jpgStart, newJpgByte, 0, headData.jpgSize); // tmpBytesからnewJPGByteへデータをコピー
                            msToImg.Write(newJpgByte, 0, newJpgByte.Length); // JPGデータをストリームに読み込む

                            if (cam == MainForm.Form1Instance.selectedCamera)   // カメラが選択されているときだけ画面を書き換える
                            {
                                try
                                {
                                    Font fnt = new Font(SystemFonts.MenuFont.Name, SystemFonts.MenuFont.Size * 2, SystemFonts.MenuFont.Style);
                                    Image im = new Bitmap(msToImg);

                                    if (MainForm.Form1Instance.cameraSettings[cam].inversePicture == true)
                                    {
                                        im.RotateFlip(RotateFlipType.Rotate180FlipNone);    // 反転フラグが立っている場合は画像を180°回転させて表示する
                                    }
                                    if (MainForm.Form1Instance.cameraSettings[cam].overlayString == true)
                                    {
                                        Graphics g = Graphics.FromImage(im);
                                        int baseColumn = 2;
                                        int baseRow = 2;
                                        g.DrawString(MainForm.Form1Instance.cameraSettings[cam].overlayStringData, fnt, Brushes.Black, baseColumn - 1, baseRow - 1);
                                        g.DrawString(MainForm.Form1Instance.cameraSettings[cam].overlayStringData, fnt, Brushes.Black, baseColumn - 1, baseRow + 1);
                                        g.DrawString(MainForm.Form1Instance.cameraSettings[cam].overlayStringData, fnt, Brushes.Black, baseColumn + 1, baseRow - 1);
                                        g.DrawString(MainForm.Form1Instance.cameraSettings[cam].overlayStringData, fnt, Brushes.Black, baseColumn + 1, baseRow + 1);
                                        g.DrawString(MainForm.Form1Instance.cameraSettings[cam].overlayStringData, fnt, Brushes.White, baseColumn, baseRow);
                                        g.Dispose();
                                    }
                                    MainForm.Form1Instance.screen = im;   // JPGデータを表示
                                }
                                catch (Exception gex)
                                {
                                    System.Diagnostics.Debug.WriteLine(gex);
                                }
                            }

                            msToImg.SetLength(0);                                                               //ストリームを一旦クリア
                            msToImg.Write(tmpBytes, headData.jpgEnd+2, tmpBytes.Length - (headData.jpgEnd+2));  // 残りのデータを保存する(次に来るデータと結合するため)
                            tmpBytes = msToImg.ToArray();
                        }

                        msToImg.SetLength(0);
                        newJpgByte = null;
                        notReceived = 0;
                        Thread.Sleep(10);
                    }
                }
                else
                {
                    notReceived = notReceived + 1;
                }

                if (notReceived > 100)
                {
                    hWebReq.Abort();
                    goto doloop;
                }

            };
            hWebReq.Abort();    // MotionJpeg取得終了したら忘れずにHTTP RequestをAbortしておく
        }


        // サムネイル映像を更新する関数
        private void getJPG(string ipAddr, int portNum, CookieContainer cookie_, int cam_)
        {
            int cam = cam_;
            httpClass htC = new httpClass();
            HttpWebRequest hWebReq;

            string url = "http://" + ipAddr + ":" + portNum + httpClass.URL_GET_JPG;

            hWebReq = (HttpWebRequest)HttpWebRequest.Create(url);
            hWebReq.Method = "GET";
            hWebReq.UserAgent = "UDP Sound Receiver";
            hWebReq.Timeout = httpClass.STREAM_TIMEOUT;
            hWebReq.KeepAlive = false;
            hWebReq.CookieContainer = cookie_;

            Stream objStream = null;

            try
            {
                objStream = hWebReq.GetResponse().GetResponseStream();
            }
            catch (Exception ex){
                Debug.WriteLine("サムネイル更新エラー:" + ex);
            }

            if (objStream != null)
            {
                Image im = createThumb(Image.FromStream(objStream),constClass.THUMB_WIDTH,constClass.THUMB_HEIGHT);

                if (MainForm.Form1Instance.cameraSettings[cam].inversePicture == true) {
                    im.RotateFlip(RotateFlipType.Rotate180FlipNone);
                }

                switch (cam)
                {
                    case 0:
                        MainForm.Form1Instance.thumbCam1 = im;
                        break;
                    case 1:
                        MainForm.Form1Instance.thumbCam2 = im;
                        break;
                    case 2:
                        MainForm.Form1Instance.thumbCam3 = im;
                        break;
                    case 3:
                        MainForm.Form1Instance.thumbCam4 = im;
                        break;
                    default:
                        break;
                }
            }

        }


        // ヘッダーのバイト列からJPGのサイズを抜き出し、返す関数
        private headerDatas getDatas_from_header(byte[] inBytes_)
        {
            headerDatas headData = new headerDatas();

            string header = Encoding.ASCII.GetString(inBytes_);
            string[] headers;

            headData.headerStart = header.IndexOf("--endof");

            if (headData.headerStart >= 0)
            {
                // ヘッダからJPGのサイズ(content-length)を取得
                header = header.Substring(headData.headerStart, HEADER_LENGTH_MAX);
                headers = header.Replace("\r\n", "\n").Split('\n'); // ヘッダを改行単位で各要素に分離する
                headData.headerSize = headers[0].Length + headers[1].Length + headers[2].Length + headers[3].Length + 8; // ヘッダのサイズを取得(1行につき2バイトCrLfが追加になる)
                headData.jpgStart = headData.headerStart + headData.headerSize;    // JPGデータの開始位置を取得
                headData.jpgSize = int.Parse(headers[2].Substring(headers[2].IndexOf(":") + 2)); // JPGデータのサイズを取得
                headData.jpgEnd = headData.jpgStart + headData.jpgSize;
                headData.Valid = true;
            }
            else
            {
                headData.Valid = false;
            }

            return headData;
        }


        // 画像をリサイズしてサムネイルを作成する関数
        private Image createThumb(Image im, int wid, int hei)
        {

            Bitmap bm = new Bitmap(wid, hei);
            Graphics gra = Graphics.FromImage(bm);

            gra.FillRectangle(new SolidBrush(Color.White), 0, 0, wid, hei); // 画面を一旦白で塗りつぶす

            float dw = im.Width * (float)wid / (float)im.Width;
            float dh = im.Height * (float)hei / (float)im.Height;

            gra.DrawImage(im, (wid - dw) / 2, (hei - dh) / 2, dw, dh);
            gra.Dispose();

            return bm;

        }


    }
}
