﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.DirectX.DirectSound;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.IO;
using System.Media;

namespace WindowsFormsApplication1
{
    class wavClass
    {

        // ストリームから受け取ったデータから、Wavのヘッダ情報を取り出すための構造体
        public struct headerDatas
        {
            public bool Valid;
            public int headerStart;
            public int headerSize;
            public int wavStart;
            public int wavSize;
            public int wavEnd;
        }


        public const int HEADER_LENGTH_MAX = 80;           // ヘッダーの最大長さ(実際は70以下だが、とりあえず80)
        public const int WAV_CHANNELS = 1;                 // WAVのチャンネル数(モノラル)
        public const int WAV_SAMPLES_PER_SEC = 16000;      // WAVのサンプリングレート
        public const int WAV_BITS_PER_SAMPLE = 16;         // WAVの量子化ビット数
        public const int WAV_PLAY_BUFFRING_COUNT = 1;      // 再生バッファのバッファ回数
        public const int WAV_INIT_BUFFRING_COUNT = 1;      // 初回バッファリングの回数
        public const int WAV_INIT_BUFFRING_INTERVAL = 0;   // 初回バッファリングの間隔
        public const int WAV_BUFFRING_INTERVAL = 0;        // ２回目以降のバッファリングの間隔
        public const int SECONDARY_BUFFER_SECONDS = 1;     // セカンダリバッファのバッファサイズ(秒)


        // 音声を再生する(通常は外部からはこちらを使用する)
        public void getLpcm() { 
            int cam = MainForm.Form1Instance.selectedCamera;
            string ipAddr = MainForm.Form1Instance.cameraSettings[cam].ipAddress;
            string port = MainForm.Form1Instance.cameraSettings[cam].portNumber;
            CookieContainer cookie = MainForm.Form1Instance.cookieContainers[cam];

            getLpcm(ipAddr, port, cookie, cam);
        }



        // 通常外部からは使用しない関数 ////////////////////////////////////
        // 音声を再生する
        public void getLpcm(string ipAddr, string portNum, CookieContainer cookie, int cam)
        {
            HttpWebRequest hWebReq;
            Stream objStream;
            MemoryStream ms2 = new MemoryStream();
            MemoryStream msToData = new MemoryStream();
            MemoryStream msPlay = new MemoryStream();

            Microsoft.DirectX.DirectSound.Device dev;
            Microsoft.DirectX.DirectSound.Buffer pbB;

            BufferDescription pbBD = new BufferDescription(); // 再生内容を指定 //
            headerDatas headData;

            // 再生デバイスの準備 //
            dev = new Device();
            dev.SetCooperativeLevel(MainForm.Form1Instance.handler, CooperativeLevel.Priority); // 再生の優先度を設定 //
            pbBD = createBufferDescription(createWavFormat(WAV_CHANNELS, WAV_SAMPLES_PER_SEC, WAV_BITS_PER_SAMPLE));
            pbBD.GlobalFocus = true;
            pbB = new SecondaryBuffer(pbBD, dev);
            pbB.Volume = 0;            // ボリューム (dB値なので0が最大)

            // 受信コード
            byte[] headBytes = new byte[171];
            byte[] tmpBytes = new byte[81920];
            byte[] initByte = { };
            byte[] nsBytes = new byte[20480];   // NetworkStream を受け取る配列
            byte[] nstmpBytes = new byte[2048]; // NetworkStream を受け取る配列(getResponseStreamから受け取ったあとは、MemoryStreamに一旦蓄積、そのあとnsBytesに移す
            byte[] newWavByte;
            int resSize;
            int pbBcount = 0;
            byte[] pbBcountBuffer = initByte;
            bool isFirstTime = true;

            hWebReq = (HttpWebRequest)HttpWebRequest.Create("http://" + ipAddr + ":" + portNum + httpClass.URL_LPCM);
            hWebReq.CookieContainer = cookie;

            hWebReq.Method = "GET";
            hWebReq.UserAgent = "C2N Viewer Receiver";
            hWebReq.Timeout = 5000;
            hWebReq.KeepAlive = true;

            tmpBytes = initByte;


            // 初回バッファリング(
            resSize = 0;
            for (int i = 0; i < WAV_INIT_BUFFRING_COUNT; i++)     // カウント数分のデータをバッファリングする
            {
                try
                {
                    objStream = hWebReq.GetResponse().GetResponseStream();  // ストリームを取得
                    resSize = objStream.Read(nstmpBytes, 0, nstmpBytes.Length);   // バイト配列に読み取り
                }
                catch (WebException wex)
                {
                    System.Diagnostics.Debug.WriteLine(wex);
                }

                if (resSize > 0)
                {
                    ms2.Write(nstmpBytes, 0, resSize);
                }
                Thread.Sleep(WAV_INIT_BUFFRING_INTERVAL);
            }


            while (MainForm.Form1Instance.receiveFlags[cam] == true)
            {

                while (cam != MainForm.Form1Instance.selectedCamera | MainForm.Form1Instance.cameraSettings[cam].playSound == false)    // カメラが選択されていないか、、音声出力フラグが立っていない間は音声出力の処理をしない
                {
                    tmpBytes = initByte;
                    pbB.Stop();

                    if (MainForm.Form1Instance.receiveFlags[cam] == false) {
                        hWebReq.Abort();
                        return; 
                    }
                    Thread.Sleep(50);
                }

                if (isFirstTime == false){

                    resSize = 0;
                    ms2.SetLength(0);

                    hWebReq.CookieContainer = cookie;

                    try {
                        objStream = hWebReq.GetResponse().GetResponseStream();  // ストリームを取得
                        resSize = objStream.Read(nstmpBytes, 0, nstmpBytes.Length);   // バイト配列に読み取り
                    } catch (WebException wex) {
                        System.Diagnostics.Debug.WriteLine(wex);
                        resSize = 0;
                    }

                    if (resSize > 0) {
                        ms2.Write(nstmpBytes, 0, resSize);
                    }
                
                }

                nsBytes = ms2.ToArray();    // バッファリングしたデータを配列に変換

                if (nsBytes.Length != 0)
                {
                    tmpBytes = tmpBytes.Concat(nsBytes).ToArray();  // 前に使わずに残っていたデータと、新しく受信したデータを結合する
                    headData = getDatas_from_header(tmpBytes); // バイトデータからヘッダー情報を読み取る

                    if (headData.Valid == true)
                    {
                        if ((tmpBytes.Length > headData.wavEnd) & (headData.wavSize != 0))
                        {
                            newWavByte = new byte[headData.wavSize];                  // Wavデータの格納先
                            Array.Copy(tmpBytes, headData.wavStart, newWavByte, 0, headData.wavSize); // tmpBytesからnewWavByteへデータをコピー

                            msToData.Write(tmpBytes, headData.wavEnd+2, tmpBytes.Length - (headData.wavEnd+2));    // 残りのデータを保存する(次に来るデータと結合するため)

                            tmpBytes = msToData.ToArray();

                            if (pbBcount > WAV_PLAY_BUFFRING_COUNT) // 再生を安定させるため、データをバッファする
                            {
                                pbBcountBuffer = msPlay.ToArray();
                               
                                pbB.Write(pbB.WritePosition, pbBcountBuffer, LockFlag.None); //SecondaryBufferにデータを書き込み
                                pbBcountBuffer = initByte;
                                pbBcount = 0;
                                msPlay.SetLength(0);

                            } else {

                                msPlay.Write(newWavByte, 0, headData.wavSize);
                                pbBcount = pbBcount + 1;

                            }

                            if (cam != MainForm.Form1Instance.selectedCamera) {
                                pbB.Stop();
                            } else {
                                if (pbB.Status.Playing == false)
                                {
                                    pbB.Play(0, BufferPlayFlags.Default);
                                }
                            }
                        }
                    }
                    else {
                        System.Diagnostics.Debug.WriteLine("");
                    }

                    msToData.SetLength(0);
                    newWavByte = null;
                    isFirstTime = false;
                    Thread.Sleep(WAV_BUFFRING_INTERVAL);
                }
            }
            pbB.Stop();
            hWebReq.Abort();
        }


        // wavフォーマットを生成する関数
        public WaveFormat createWavFormat(short channels_, int samplesPerSecond_, short bitsPerSample_)
        {
            WaveFormat wf = new WaveFormat();

            wf.FormatTag = WaveFormatTag.Pcm;
            wf.Channels = channels_;
            wf.SamplesPerSecond = samplesPerSecond_;
            wf.BitsPerSample = bitsPerSample_;
            wf.BlockAlign = (short)(wf.Channels * (wf.BitsPerSample / (short)8));
            wf.AverageBytesPerSecond = wf.BlockAlign * wf.SamplesPerSecond;         // 1秒あたりのバイト数
            return wf;
        }


        // 再生内容(description)を生成する関数
        public BufferDescription createBufferDescription(WaveFormat wf)
        {
            BufferDescription bd = new BufferDescription();

            bd.BufferBytes = wf.AverageBytesPerSecond * SECONDARY_BUFFER_SECONDS; //バッファサイズの指定(とりあえず2秒分)
            bd.ControlVolume = true;  // ボリュームコントロールを可能にする
            bd.Format = wf;           // Wavフォーマットを指定
            bd.GlobalFocus = true;    // ウィンドウが非アクティブの場合でも音を再生する
            return bd;
        }


        // ヘッダーのバイト列からWavのサイズを抜き出し、返す関数
        private headerDatas getDatas_from_header(byte[] inBytes_)
        {
            headerDatas headData = new headerDatas();

            string header = Encoding.ASCII.GetString(inBytes_);
            string[] headers;

            headData.headerStart = header.IndexOf("--endof");

            if (headData.headerStart >= 0)
            {
                // ヘッダからWAVのサイズ(content-length)を取得
                header = header.Substring(headData.headerStart, HEADER_LENGTH_MAX);
                headers = header.Replace("\r\n", "\n").Split('\n'); // ヘッダを改行単位で各要素に分離する
                headData.headerSize = headers[0].Length + headers[1].Length + headers[2].Length + headers[3].Length + 8; // ヘッダのサイズを取得(1行につき2バイトCrLfが追加になる)
                headData.wavStart = headData.headerStart + headData.headerSize;    // WAVデータの開始位置を取得
                headData.wavSize = int.Parse(headers[2].Substring(headers[2].IndexOf(":") + 2)); // WAVデータのサイズを取得
                headData.wavEnd = headData.wavStart + headData.wavSize;
                headData.Valid = true;
            }
            else
            {
                headData.Valid = false;
            }
            return headData;
        }



    }
}
